# coding=utf-8
"""
This file only exists to not break extensions which have

    <dependency type="executable" location="extensions">inkex.py</dependency>

This module should never be imported, Python should automatically find
"inkex/__init__.py" before finding this file.
"""

raise Exception("FIXME: Python imported " + __file__)



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2011 Felipe Correa da Silva Sanches
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

from next_glyph_layer import NextLayer

class PreviousLayer(NextLayer):
    """Like next glyph layer, but for the previous"""
    @staticmethod
    def process_glyphs(glyphs, current):
        glyphs[current].set("style", "display:none")
        glyphs[current-1].set("style", "display:inline")

if __name__ == '__main__':
    PreviousLayer().run()



# coding=utf-8
#
# Copyright (C) 2007 Martin Owens
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
#
"""
Python barcode renderer for Code39 barcodes. Designed for use with Inkscape.
"""

from .Base import Barcode

ENCODE = {
    '0': '000110100',
    '1': '100100001',
    '2': '001100001',
    '3': '101100000',
    '4': '000110001',
    '5': '100110000',
    '6': '001110000',
    '7': '000100101',
    '8': '100100100',
    '9': '001100100',
    'A': '100001001',
    'B': '001001001',
    'C': '101001000',
    'D': '000011001',
    'E': '100011000',
    'F': '001011000',
    'G': '000001101',
    'H': '100001100',
    'I': '001001100',
    'J': '000011100',
    'K': '100000011',
    'L': '001000011',
    'M': '101000010',
    'N': '000010011',
    'O': '100010010',
    'P': '001010010',
    'Q': '000000111',
    'R': '100000110',
    'S': '001000110',
    'T': '000010110',
    'U': '110000001',
    'V': '011000001',
    'W': '111000000',
    'X': '010010001',
    'Y': '110010000',
    'Z': '011010000',
    '-': '010000101',
    '*': '010010100',
    '+': '010001010',
    '$': '010101000',
    '%': '000101010',
    '/': '010100010',
    '.': '110000100',
    ' ': '011000100',
}


class Code39(Barcode):
    """Convert a text into string binary of black and white markers"""

    def encode(self, text):
        self.text = text.upper()
        result = ''
        # It is possible for us to encode code39
        # into full ascii, but this feature is
        # not enabled here
        for char in '*' + self.text + '*':
            if char not in ENCODE:
                char = '-'
            result = result + ENCODE[char] + '0'

        # Now we need to encode the code39, best read
        # the code to understand what it's up to:
        encoded = ''
        colour = '1'  # 1 = Black, 0 = White
        for data in result:
            if data == '1':
                encoded = encoded + colour + colour
            else:
                encoded = encoded + colour
            colour = colour == '1' and '0' or '1'

        return encoded



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2015 ~suv <suv-sf@users.sf.net>
# Copyright (C) 2010 Alvin Penner
# Copyright (C) 2006 Georg Wiora
# Copyright (C) 2006 Nathan Hurst
# Copyright (C) 2005 Aaron Spike, aaron@ekips.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
This extension module can measure arbitrary path and object length
It adds text to the selected path containing the length in a given unit.
Area and Center of Mass calculated using Green's Theorem:
http://mathworld.wolfram.com/GreensTheorem.html
"""

import inkex

from inkex import TextElement, TextPath, Tspan
from inkex.bezier import csparea, cspcofm, csplength

class MeasureLength(inkex.EffectExtension):
    """Measure the length of selected paths"""
    def add_arguments(self, pars):
        pars.add_argument("--type", dest="mtype", default="length",\
            help="Type of measurement")
        pars.add_argument("--method", type=self.arg_method(), default=self.method_textonpath,\
            help="Text Orientation method")
        pars.add_argument("--presetFormat", default="TaP_start", help="Preset text layout")
        pars.add_argument("--startOffset", default="custom", help="Text Offset along Path")
        pars.add_argument("--startOffsetCustom", type=int, default=50,\
            help="Text Offset along Path")
        pars.add_argument("--anchor", default="start", help="Text Anchor")
        pars.add_argument("--position", default="start", help="Text Position")
        pars.add_argument("--angle", type=float, default=0, help="Angle")
        pars.add_argument("-f", "--fontsize", type=int, default=20,\
            help="Size of length label text in px")
        pars.add_argument("-o", "--offset", type=float, default=-6,\
            help="The distance above the curve")
        pars.add_argument("-u", "--unit", default="mm",\
            help="The unit of the measurement")
        pars.add_argument("-p", "--precision", type=int, default=2,\
            help="Number of significant digits after decimal point")
        pars.add_argument("-s", "--scale", type=float, default=1.1,\
            help="Scale Factor (Drawing:Real Length)")

    def effect(self):
        # get number of digits
        prec = int(self.options.precision)
        scale = self.svg.unittouu('1px')  # convert to document units
        self.options.offset *= scale
        factor = 1.0

        if self.svg.get('viewBox'):
            factor = self.svg.scale / self.svg.unittouu('1px')
            self.options.fontsize /= factor

        factor *= scale / self.svg.unittouu('1' + self.options.unit)

        # loop over all selected paths
        filtered = self.svg.selection.filter(inkex.PathElement)
        if not filtered:
            raise inkex.AbortExtension("Please select at least one path object.")
        for node in filtered:
            csp = node.path.transform(node.composed_transform()).to_superpath()
            inverse_parent_transform = - node.getparent().composed_transform()
            if self.options.mtype == "length":
                slengths, stotal = csplength(csp)
                self.group = node.getparent().add(TextElement())
            elif self.options.mtype == "area":
                stotal = abs(csparea(csp) * factor * self.options.scale)
                self.group = node.getparent().add(TextElement())
            else:
                try:
                    xc, yc = cspcofm(csp)
                except ValueError as err:
                    raise inkex.AbortExtension(str(err))
                self.group = node.getparent().add(inkex.PathElement())
                self.group.set('id', 'MassCenter_' + node.get('id'))
                self.add_cross(self.group, xc, yc, scale)
                self.group.transform = inverse_parent_transform
                continue
            # Format the length as string
            val = round(stotal * factor * self.options.scale, prec)
            # Transform the result back
            self.options.method(node, str(val))

    def method_textonpath(self, node, lenstr):
        startOffset = self.options.startOffset
        if startOffset == "custom":
            startOffset = str(self.options.startOffsetCustom) + '%'
        if self.options.mtype == "length":
            self.add_textonpath(self.group, 0, 0, lenstr + ' ' + self.options.unit, node, self.options.anchor, startOffset, self.options.offset)
        else:
            self.add_textonpath(self.group, 0, 0, lenstr + ' ' + self.options.unit + '^2', node, self.options.anchor, startOffset, self.options.offset)

    def method_fixedtext(self, node, lenstr):
        _id = node.get('id')
        csp = node.path.transform(node.composed_transform()).to_superpath()
        if self.options.position == "mass":
            tx, ty = cspcofm(csp)
            anchor = 'middle'
        elif self.options.position == "center":
            bbox = node.bounding_box(True)
            tx, ty = bbox.center
            anchor = 'middle'
        else:  # default
            tx = csp[0][0][1][0]
            ty = csp[0][0][1][1]
            anchor = 'start'
        if self.options.mtype == "length":
            self.add_fixedtext(self.group, tx, ty, lenstr + ' ' + self.options.unit, anchor, -int(self.options.angle), self.options.offset + self.options.fontsize / 2)
        else:
            self.add_fixedtext(self.group, tx, ty, lenstr + ' ' + self.options.unit + '^2', anchor, -int(self.options.angle), -self.options.offset + self.options.fontsize / 2)

    def method_presets(self, node, lenstr):
        """A preset option for alignments"""
        preset_dict = {
            'default_cofm':   [None, None, None, None, None],
            'default_length': [self.method_textonpath, "50%", "start", None, None],
            'TaP_start':      [self.method_textonpath, "0%", "start", None, None],
            'TaP_middle':     [self.method_textonpath, "50%", "middle", None, None],
            'TaP_end':        [self.method_textonpath, "100%", "end", None, None],
            'default_area':   [self.method_fixedtext, None, None, "start", 0.0],
            'FT_start':       [self.method_fixedtext, None, None, "start", 0.0],
            'FT_bbox':        [self.method_fixedtext, None, None, "center", 0.0],
            'FT_mass':        [self.method_fixedtext, None, None, "mass", 0.0],
        }

        if self.options.presetFormat == "default":
            current_preset = 'default_' + self.options.mtype
        else:
            current_preset = self.options.presetFormat

        self.options.startOffset = preset_dict[current_preset][1]
        self.options.anchor = preset_dict[current_preset][2]
        self.options.position = preset_dict[current_preset][3]
        self.options.angle = preset_dict[current_preset][4]
        method = preset_dict[current_preset][0]
        if method is not None:
            return method(node, lenstr)

    def add_cross(self, node, x, y, scale):
        l = 3 * scale  # 3 pixels in document units
        node.set('d', 'm %s,%s %s,0 %s,0 m %s,%s 0,%s 0,%s' % (str(x - l), str(y), str(l), str(l), str(-l), str(-l), str(l), str(l)))
        node.set('style', 'stroke:#000000;fill:none;stroke-width:%s' % str(0.5 * scale))

    def add_textonpath(self, node, x, y, text, _node, anchor, startOffset, dy=0):
        new = node.add(TextPath())
        s = {'text-align': 'center', 'vertical-align': 'bottom',
             'text-anchor': anchor, 'font-size': str(self.options.fontsize),
             'fill-opacity': '1.0', 'stroke': 'none',
             'font-weight': 'normal', 'font-style': 'normal', 'fill': '#000000'}
        new.style = s
        new.href = _node
        new.set('startOffset', startOffset)
        new.set('dy', str(dy))  # dubious merit
        # new.append(tp)
        if text[-2:] == "^2":
            new.append(Tspan.superscript("2"))
            new.text = str(text)[:-2]
        else:
            new.text = str(text)
        # node.set('transform','rotate(180,'+str(-x)+','+str(-y)+')')
        node.set('x', str(x))
        node.set('y', str(y))

    def add_fixedtext(self, node, x, y, text, anchor, angle, dy=0):
        new = node.add(Tspan())
        new.set('sodipodi:role', 'line')
        s = {'text-align': 'center', 'vertical-align': 'bottom',
             'text-anchor': anchor, 'font-size': str(self.options.fontsize),
             'fill-opacity': '1.0', 'stroke': 'none',
             'font-weight': 'normal', 'font-style': 'normal', 'fill': '#000000'}
        new.style = s
        new.set('dy', str(dy))
        if text[-2:] == "^2":
            new.append(Tspan.superscript("2"))
            new.text = str(text)[:-2]
        else:
            new.text = str(text)
        node.set('x', str(x))
        node.set('y', str(y))
        node.set('transform', 'rotate(%s, %s, %s)' % (angle, x, y))
        node.transform = - node.getparent().composed_transform() * node.transform

if __name__ == '__main__':
    MeasureLength().run()



# -*- coding: utf-8 -*-
#
# Copyright (c) 2020 Martin Owens <doctormo@gmail.com>
#                    Sergei Izmailov <sergei.a.izmailov@gmail.com>
#                    Thomas Holder <thomas.holder@schrodinger.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
# pylint: disable=arguments-differ
"""
Interface for all shapes/polygons such as lines, paths, rectangles, circles etc.
"""

from ..paths import Path
from ..transforms import Transform, ImmutableVector2d, Vector2d

from ._utils import addNS
from ._base import ShapeElement

class PathElementBase(ShapeElement):
    """Base element for path based shapes"""
    get_path = lambda self: self.get('d')

    @classmethod
    def new(cls, path, **attrs):
        return super().new(d=Path(path), **attrs)

    def set_path(self, path):
        """Set the given data as a path as the 'd' attribute"""
        self.set('d', str(Path(path)))

    def apply_transform(self):
        """Apply the internal transformation to this node and delete"""
        if 'transform' in self.attrib:
            self.path = self.path.transform(self.transform)
            self.set('transform', Transform())

    @property
    def original_path(self):
        """Returns the original path if this is a LPE, or the path if not"""
        return Path(self.get('inkscape:original-d', self.path))

    @original_path.setter
    def original_path(self, path):
        if addNS('inkscape:original-d') in self.attrib:
            self.set('inkscape:original-d', str(Path(path)))
        else:
            self.path = path


class PathElement(PathElementBase):
    """Provide a useful extension for path elements"""
    tag_name = 'path'

    @classmethod
    def arc(cls, center, rx, ry=None, **kw): # pylint: disable=invalid-name
        """Generate a sodipodi arc (special type)"""
        others = [(name, kw.pop(name, None)) for name in ('start', 'end', 'open')]
        elem = cls(**kw)
        elem.set('sodipodi:cx', center[0])
        elem.set('sodipodi:cy', center[1])
        elem.set('sodipodi:rx', rx)
        elem.set('sodipodi:ry', ry or rx)
        elem.set('sodipodi:type', 'arc')
        for name, value in others:
            if value is not None:
                elem.set('sodipodi:'+name, str(value).lower())
        return elem

    @classmethod
    def star(cls, center, radi, sides, rounded=None):
        """Generate a sodipodi start (special type)"""
        elem = cls()
        elem.set('sodipodi:cx', center[0])
        elem.set('sodipodi:cy', center[1])
        elem.set('sodipodi:r1', radi[0])
        elem.set('sodipodi:r2', radi[1])
        elem.set('sodipodi:arg1', 0.85)
        elem.set('sodipodi:arg2', 1.3)
        elem.set('sodipodi:sides', sides)
        elem.set('inkscape:rounded', rounded)
        elem.set('sodipodi:type', 'star')
        return elem


class Polyline(ShapeElement):
    """Like a path, but made up of straight line segments only"""
    tag_name = 'polyline'

    def get_path(self):
        return Path('M' + self.get('points'))

    def set_path(self, path):
        points = [f'{x:g},{y:g}' for x, y in Path(path).end_points]
        self.set('points', ' '.join(points))


class Polygon(ShapeElement):
    """A closed polyline"""
    tag_name = 'polygon'
    get_path = lambda self: 'M' + self.get('points') + ' Z'


class Line(ShapeElement):
    """A line segment connecting two points"""
    tag_name = 'line'
    get_path = lambda self: 'M{0[x1]},{0[y1]} L{0[x2]},{0[y2]} Z'.format(self.attrib)

    @classmethod
    def new(cls, start, end, **attrs):
        start = Vector2d(start)
        end = Vector2d(end)
        return super().new(x1=start.x, y1=start.y,
                                    x2=end.x, y2=end.y, **attrs)


class RectangleBase(ShapeElement):
    """Provide a useful extension for rectangle elements"""
    left = property(lambda self: self.uutounit(self.get('x', '0'), 'px'))
    top = property(lambda self: self.uutounit(self.get('y', '0'), 'px'))
    right = property(lambda self: self.left + self.width)
    bottom = property(lambda self: self.top + self.height)
    width = property(lambda self: self.uutounit(self.get('width', '0'), 'px'))
    height = property(lambda self: self.uutounit(self.get('height', '0'), 'px'))
    rx = property(lambda self: self.uutounit(self.get('rx', self.get('ry', 0.0)), 'px'))
    ry = property(lambda self: self.uutounit(self.get('ry', self.get('rx', 0.0)), 'px')) # pylint: disable=invalid-name

    def get_path(self):
        """Calculate the path as the box around the rect"""
        if self.rx:
            rx, ry = self.rx, self.ry # pylint: disable=invalid-name
            return 'M {1},{0.top}'\
                   'L {2},{0.top}    A {0.rx},{0.ry} 0 0 1 {0.right},{3}'\
                   'L {0.right},{4}  A {0.rx},{0.ry} 0 0 1 {2},{0.bottom}'\
                   'L {1},{0.bottom} A {0.rx},{0.ry} 0 0 1 {0.left},{4}'\
                   'L {0.left},{3}   A {0.rx},{0.ry} 0 0 1 {1},{0.top} z'\
                .format(self, self.left + rx, self.right - rx, self.top + ry, self.bottom - ry)

        return 'M {0.left},{0.top} h{0.width}v{0.height}h{1} z'.format(self, -self.width)


class Rectangle(RectangleBase):
    """Provide a useful extension for rectangle elements"""
    tag_name = 'rect'

    @classmethod
    def new(cls, left, top, width, height, **attrs):
        return super().new(x=left, y=top, width=width, height=height, **attrs)


class EllipseBase(ShapeElement):
    """Absorbs common part of Circle and Ellipse classes"""

    def get_path(self):
        """Calculate the arc path of this circle"""
        rx, ry = self._rxry()
        cx, y = self.center.x, self.center.y - ry
        return ('M {cx},{y} '
                'a {rx},{ry} 0 1 0 {rx}, {ry} '
                'a {rx},{ry} 0 0 0 -{rx}, -{ry} z'
                ).format(cx=cx, y=y, rx=rx, ry=ry)

    @property
    def center(self):
        return ImmutableVector2d(self.uutounit(self.get('cx', '0')), self.uutounit(self.get('cy', '0')))

    @center.setter
    def center(self, value):
        value = Vector2d(value)
        self.set("cx", value.x)
        self.set("cy", value.y)

    def _rxry(self):
        # type: () -> Vector2d
        """Helper function """
        raise NotImplementedError()

    @classmethod
    def new(cls, center, radius, **attrs):
        circle = super().new(**attrs)
        circle.center = center
        circle.radius = radius
        return circle


class Circle(EllipseBase):
    """Provide a useful extension for circle elements"""
    tag_name = 'circle'

    @property
    def radius(self):
        return self.uutounit(self.get('r', '0'), 'px')

    @radius.setter
    def radius(self, value):
        self.set("r", self.unittouu(value))

    def _rxry(self):
        r = self.radius
        return Vector2d(r, r)


class Ellipse(EllipseBase):
    """Provide a similar extension to the Circle interface"""
    tag_name = 'ellipse'

    @property
    def radius(self):
        return ImmutableVector2d(self.uutounit(self.get('rx', '0')), self.uutounit(self.get('ry', '0')))

    @radius.setter
    def radius(self, value):
        value = Vector2d(value)
        self.set("rx", str(value.x))
        self.set("ry", str(value.y))

    def _rxry(self):
        return self.radius



# coding=utf-8
#
# Copyright (C) 2010 Martin Owens
#
# Thanks to Lineaire Chez of Inkbar ( www.inkbar.lineaire.net )
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
#
"""
Python barcode renderer for EAN13 barcodes. Designed for use with Inkscape.
"""

from .BaseEan import EanBarcode


class Ean13(EanBarcode):
    """Provide an Ean13 barcode generator"""
    name = 'ean13'
    extras = {2: 'Ean2', 5: 'Ean5'}
    checks = [13]
    lengths = [12]

    def _encode(self, num, guide=False):
        """Encode an ean13 barcode"""
        self.text = self.space(num[0:1], 4, num[1:7], 5, num[7:], 7)
        if guide:
            self.text = self.text[:-4] + '>'
        return self.enclose(
                self.encode_interleaved(num[0], num[1:7]),
                self.encode_right(num[7:])
        )



#!/usr/bin/env python
"""Remove colors"""

import inkex

class Desaturate(inkex.ColorExtension):
    """Remove color but maintain intesity"""
    def modify_color(self, name, color):
        lum = (max(color.red, color.green, color.blue) \
             + min(color.red, color.green, color.blue)) // 2
        return inkex.Color((int(round(lum)), int(round(lum)), int(round(lum))))

if __name__ == '__main__':
    Desaturate().run()



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2007 Terry Brown, terry_n_brown@yahoo.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

from math import atan2, degrees

import inkex
from inkex import ClipPath, Filter


class Edge3D(inkex.EffectExtension):
    """Generate a 3d edge"""
    def add_arguments(self, pars):
        pars.add_argument('--angle', type=float, default=45.0,
                          help='angle of illumination, clockwise, 45 = upper right')
        pars.add_argument('--stddev', type=float, default=5.0, help='Gaussian Blur stdDeviation')
        pars.add_argument('--blurheight', type=float, default=2.0, help='Gaussian Blur height')
        pars.add_argument('--blurwidth', type=float, default=2.0, help='Gaussian Blur width')
        pars.add_argument('--shades', type=int, default=2, help="Number of shades")
        pars.add_argument('--bw', type=inkex.Boolean, help="Black and white")
        pars.add_argument('--thick', type=float, default=10.0, help='stroke-width for pieces')

    def angle_between(self, start, end, angle):
        """Return true if angle (degrees, clockwise, 0 = up/north) is between
           angles start and end"""

        def f(x):
            """Add 360 to x if x is less than 0"""
            if x < 0:
                return x + 360.
            return x

        # rotate all inputs by start, => start = 0
        a = f(f(angle) - f(start))
        e = f(f(end) - f(start))
        return a < e

    def effect(self):
        """Check each internode to see if it's in one of the wedges
           for the current shade.  shade is a floating point 0-1 white-black"""
        # size of a wedge for shade i, wedges come in pairs
        delta = 360. / self.options.shades / 2.
        for node in self.svg.selection.filter(inkex.PathElement):
            array = node.path.to_arrays()
            group = None
            filt = None
            for shade in range(0, self.options.shades):
                if self.options.bw and 0 < shade < self.options.shades - 1:
                    continue
                start = [self.options.angle - delta * (shade + 1)]
                end = [self.options.angle - delta * shade]
                start.append(self.options.angle + delta * shade)
                end.append(self.options.angle + delta * (shade + 1))
                level = float(shade) / float(self.options.shades - 1)
                last = []
                result = []
                for cmd, params in array:
                    if cmd == 'Z':
                        last = []
                        continue
                    if last:
                        if cmd == 'V':
                            point = [last[0], params[-2:][0]]
                        elif cmd == 'H':
                            point = [params[-2:][0], last[1]]
                        else:
                            point = params[-2:]
                        ang = degrees(atan2(point[0] - last[0], point[1] - last[1]))
                        if (self.angle_between(start[0], end[0], ang) or \
                            self.angle_between(start[1], end[1], ang)):
                            result.append(('M', last))
                            result.append((cmd, params))
                        ref = point
                    else:
                        ref = params[-2:]
                    last = ref
                if result:
                    if group is None:
                        group, filt = self.get_group(node)
                    new_node = group.add(node.copy())
                    new_node.path = result
                    new_node.style = 'fill:none;stroke-opacity:1;stroke-width:10'
                    new_node.style += filt
                    col = 255 - int(255. * level)
                    new_node.style['stroke'] = inkex.Color((col, col, col))

    def get_group(self, node):
        """
        make a clipped group, clip with clone of original, clipped group
        include original and group of paths.
        """
        defs = self.svg.defs
        clip = defs.add(ClipPath())
        new_node = clip.add(node.copy())
        clip_group = node.getparent().add(inkex.Group())
        group = clip_group.add(inkex.Group())
        clip_group.set('clip-path', clip.get_id(as_url=2))

        # make a blur filter reference by the style of each path
        filt = defs.add(Filter(x='-0.5', y='-0.5',\
            height=str(self.options.blurheight),\
            width=str(self.options.blurwidth)))

        filt.add_primitive('feGaussianBlur', stdDeviation=self.options.stddev)
        return group, inkex.Style(filter=filt.get_id(as_url=2))

if __name__ == '__main__':
    Edge3D().run()



# coding=utf-8
#
# Copyright (C) 2008 Stephen Silver
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
#
"""
Deprecated module for running SVG-generating commands in Inkscape extensions
"""
import os
import sys
import tempfile
from subprocess import Popen, PIPE

from inkex.deprecated import deprecate

def run(command_format, prog_name):
    """inkex.commands.call(...)"""
    svgfile = tempfile.mktemp(".svg")
    command = command_format % svgfile
    msg = None
    # ps2pdf may attempt to write to the current directory, which may not
    # be writeable, so we switch to the temp directory first.
    try:
        os.chdir(tempfile.gettempdir())
    except IOError:
        pass

    try:
        proc = Popen(command, shell=True, stdout=PIPE, stderr=PIPE)
        return_code = proc.wait()
        out = proc.stdout.read()
        err = proc.stderr.read()

        if msg is None:
            if return_code:
                msg = "{} failed:\n{}\n{}\n".format(prog_name, out, err)
            elif err:
                sys.stderr.write("{} executed but logged the following error:\n{}\n{}\n".format(prog_name, out, err))
    except Exception as inst:
        msg = "Error attempting to run {}: {}".format(prog_name, str(inst))

    # If successful, copy the output file to stdout.
    if msg is None:
        if os.name == 'nt':  # make stdout work in binary on Windows
            import msvcrt
            msvcrt.setmode(sys.stdout.fileno(), os.O_BINARY)
        try:
            with open(svgfile, "rb") as fhl:
                sys.stdout.write(fhl.read().decode(sys.stdout.encoding))
        except IOError as inst:
            msg = "Error reading temporary file: {}".format(str(inst))

    try:
        # Clean up.
        os.remove(svgfile)
    except (IOError, OSError):
        pass

    # Output error message (if any) and exit.
    return msg




# coding=utf-8
#
# Copyright (C) 2007 Martin Owens
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Python barcode renderer for RM4CC barcodes. Designed for use with Inkscape.
"""

from .Base import Barcode

map = {
    '(': '25',
    ')': '3',
    '0': '05053535',
    '1': '05152535',
    '2': '05153525',
    '3': '15052535',
    '4': '15053525',
    '5': '15152525',
    '6': '05251535',
    '7': '05350535',
    '8': '05351525',
    '9': '15250535',
    'A': '15251525',
    'B': '15350525',
    'C': '05253515',
    'D': '05352515',
    'E': '05353505',
    'F': '15252515',
    'G': '15253505',
    'H': '15352505',
    'I': '25051535',
    'J': '25150535',
    'K': '25151525',
    'L': '35050535',
    'M': '35051525',
    'N': '35150525',
    'O': '25053525',
    'P': '25152515',
    'Q': '25153505',
    'R': '35052515',
    'S': '35053505',
    'T': '35152505',
    'U': '25251515',
    'V': '25350515',
    'W': '25351505',
    'X': '35250515',
    'Y': '35251505',
    'Z': '35350505',
}

check = ['ZUVWXY', '501234', 'B6789A', 'HCDEFG', 'NIJKLM', 'TOPQRS']
(BAR_TRACK, BAR_DOWN, BAR_UP, BAR_FULL, BAR_NONE, WHITE_SPACE) = range(6)


class Rm4scc(Barcode):
    default_height = 18

    def encode(self, text):
        result = ''

        text = text.upper()
        text.replace('(', '')
        text.replace(')', '')

        text = '(' + text + self.checksum(text) + ')'

        i = 0
        for char in text:
            if char in map:
                result = result + map[char]
                i += 1

        return result

    # given a string of data, return the check character
    def checksum(self, text):
        total_lower = 0
        total_upper = 0
        for char in text:
            if char in map:
                bars = map[char][0:8:2]
                lower = 0
                upper = 0

                if int(bars[0]) & 1:
                    lower += 4
                if int(bars[1]) & 1:
                    lower += 2
                if int(bars[2]) & 1:
                    lower += 1
                if int(bars[0]) & 2:
                    upper += 4
                if int(bars[1]) & 2:
                    upper += 2
                if int(bars[2]) & 2:
                    upper += 1
            total_lower += lower % 6
            total_upper += upper % 6

        total_lower = total_upper % 6
        total_upper %= 6

        checkchar = check[total_upper][total_lower]
        return checkchar

    def get_style(self, index):
        """Royal Mail Barcodes use a completely different style"""
        result = {'width': 2, 'write': True, 'top': 0}
        if index == BAR_TRACK:  # Track Bar
            result['top'] = 6
            result['height'] = 5
        elif index == BAR_DOWN:  # Decender Bar
            result['top'] = 6
            result['height'] = 11
        elif index == BAR_UP:  # Accender Bar
            result['height'] = 11
        elif index == BAR_FULL:  # Full Bar
            result['height'] = 17
        elif index == WHITE_SPACE:  # White Space
            result['write'] = False
        return result



#!/usr/bin/env python
"""Convert to sentence case"""

import inkex

class SentenceCase(inkex.TextExtension):
    """Convert text to sentence case"""
    sentence_start = True
    was_punctuation = False

    def map_char(self, char):
        """Turn the char into a sentence using class state"""
        if char in '.!?':
            self.was_punctuation = True
        elif ((char.isspace() or self.newline) and self.was_punctuation) or self.newpar:
            self.sentence_start = True
            self.was_punctuation = False
        elif char in '")':
            pass
        else:
            self.was_punctuation = False

        if not char.isspace():
            self.newline = False
            self.newpar = False

        if self.sentence_start and char.isalpha():
            self.sentence_start = False
            return char.upper()
        elif not self.sentence_start and char.isalpha():
            return char.lower()
        return char

if __name__ == '__main__':
    SentenceCase().run()



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2011 Nikita Kitaev
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
#
"""
Simplifies SVG files in preparation for sif export.
"""

import os
import tempfile
from subprocess import PIPE, Popen

import inkex
from inkex import load_svg, Group, PathElement, ShapeElement,\
                           Anchor, Switch, SvgDocumentElement, Transform

###### Utility Classes ####################################

class MalformedSVGError(Exception):
    """Raised when the SVG document is invalid or contains unsupported features"""

    def __init__(self, value):
        self.value = value

    def __str__(self):
        return """SVG document is invalid or contains unsupported features

Error message: %s

The SVG to Synfig converter is designed to handle SVG files that were created using Inkscape. Unsupported features are most likely to occur in SVG files written by other programs.
""" % repr(self.value)


class InkscapeActionGroup(object):
    """A class for calling Inkscape to perform operations on a document"""

    def __init__(self, svg_document=None):
        self.command = ""
        self.init_args = ""
        self.has_selection = False
        self.has_action = False
        self.set_svg_document(svg_document)

    def set_svg_document(self, svg_document):
        """Set the SVG document that Inkscape will operate on"""
        self.svg_document = svg_document
        self.svg = svg_document.getroot()

    def set_init_args(self, cmd):
        """Set the initial arguments to Inkscape subprocess

        Can be used to pass additional arguments to Inkscape, or an initializer
        command (e.g. unlock all objects before proceeding).
        """
        self.init_args = cmd

    def clear(self):
        """Clear all actions"""
        self.command = ""
        self.has_action = False
        self.has_selection = False

    def verb(self, verb):
        """Run an Inkscape verb

        For a list of verbs, run `inkscape --verb-list`
        """
        if self.has_selection:
            self.command += "--verb=%s " % verb

            if not self.has_action:
                self.has_action = True

    def select_id(self, object_id):
        """Select object with given id"""
        self.command += "--select=%s " % object_id
        if not self.has_selection:
            self.has_selection = True

    def select_node(self, node):
        """Select the object represented by the SVG node

        Selection will fail if node has no id attribute
        """
        node_id = node.get("id", None)
        if node_id is None:
            raise MalformedSVGError("Node has no id")
        self.select_id(node_id)

    def select_nodes(self, nodes):
        """Select objects represented by SVG nodes

        Selection will fail if any node has no id attribute
        """
        for node in nodes:
            self.select_node(node)

    def select_xpath(self, xpath):
        """Select objects matching a given XPath expression

        Selection will fail if any matching node has no id attribute
        """
        self.select_nodes(self.svg.xpath(xpath))

    def deselect(self):
        """Deselect all objects"""
        if self.has_selection:
            self.verb("EditDeselect")
            self.has_selection = False

    def run_file(self, filename):
        """Run the actions on a specific file"""
        if not self.has_action:
            return

        cmd = self.init_args + " " + self.command + "--verb=FileSave --verb=FileQuit"
        p = Popen('inkscape "{}" {}'.format(filename, cmd), shell=True, stdout=PIPE, stderr=PIPE)
        rc = p.wait()
        f = p.stdout
        err = p.stderr

        f.close()
        err.close()

    def run_document(self):
        """Run the actions on the svg xml tree"""
        if not self.has_action:
            return self.svg_document

        # First save the document
        svgfile = tempfile.mktemp(".svg")
        self.svg_document.write(svgfile)

        # Run the action on the document
        self.run_file(svgfile)

        # Open the resulting file
        with open(svgfile, 'r') as stream:
            self.svg_document = load_svg(stream)

        # Clean up.
        try:
            os.remove(svgfile)
        except Exception:
            pass

        # Return the new document
        return self.svg_document


class SynfigExportActionGroup(InkscapeActionGroup):
    """An action group with stock commands designed for Synfig exporting"""

    def __init__(self, svg_document=None):
        InkscapeActionGroup.__init__(self, svg_document)
        self.set_init_args("--verb=UnlockAllInAllLayers")
        self.objects_to_paths()
        self.unlink_clones()

    def objects_to_paths(self):
        """Convert unsupported objects to paths"""
        # Flow roots contain rectangles inside them, so they need to be
        # converted to paths separately from other shapes
        self.select_xpath("//svg:flowRoot")
        self.verb("ObjectToPath")
        self.deselect()

        non_paths = [
            "svg:rect",
            "svg:circle",
            "svg:ellipse",
            "svg:line",
            "svg:polyline",
            "svg:polygon",
            "svg:text"
        ]

        # Build an xpath command to select these nodes
        xpath_cmd = " | ".join(["//" + np for np in non_paths])

        # Select all of these elements
        # Note: already selected elements are not deselected
        self.select_xpath(xpath_cmd)

        # Convert them to paths
        self.verb("ObjectToPath")
        self.deselect()

    def unlink_clones(self):
        """Unlink clones (remove <svg:use> elements)"""
        self.select_xpath("//svg:use")
        self.verb("EditUnlinkClone")
        self.deselect()


###### Utility Functions ##################################

### Path related

def fuse_subpaths(path_node):
    """Fuse subpaths of a path. Should only be used on unstroked paths"""
    path = path_node.path.to_arrays()

    if len(path) == 0:
        return

    i = 0
    initial_point = [path[i][1][-2], path[i][1][-1]]
    prev_end = initial_point[:]
    return_stack = []
    while i < len(path):
        # Remove any terminators: they are redundant
        if path[i][0] == "Z":
            path.remove(["Z", []])
            continue

        if path[i][0] == 'V':
            prev_end[0] = path[i][1][0]
            i += 1
            continue
        elif path[i][0] == 'H':
            prev_end[1] = path[i][1][0]
            i += 1
            continue
        elif path[1][0] != 'M' or i == 0:
            prev_end = path[i][1][-2:]
            i += 1
            continue

        # This element begins a new path - it should be a moveto
        assert (path[i][0] == 'M')

        # Swap it for a lineto
        path[i][0] = 'L'
        # If the old subpath has not been closed yet, close it
        if prev_end != initial_point:
            path.insert(i, ['L', initial_point])
            i += 1

        # Set the initial point of this subpath
        initial_point = path[i][1][-2:]

        # Append this point to the return stack
        return_stack.append(initial_point)
    # end while

    # Now pop the entire return stack
    while return_stack:
        el = ['L', return_stack.pop()]
        path.insert(i, el)
        i += 1

    path_d = str(inkex.Path(path))
    path_node.set("d", path_d)


def split_fill_and_stroke(path_node):
    """Split a path into two paths, one filled and one stroked

    Returns a the list [fill, stroke], where each is the XML element of the
    fill or stroke, or None.
    """
    style = dict(inkex.Style.parse_str(path_node.get("style", "")))

    # If there is only stroke or only fill, don't split anything
    if "fill" in style and style["fill"] == "none":
        if "stroke" not in style or style["stroke"] == "none":
            return [None, None]  # Path has neither stroke nor fill
        else:
            return [None, path_node]
    if "stroke" not in style.keys() or style["stroke"] == "none":
        return [path_node, None]


    group = Group()
    fill = group.add(PathElement())
    stroke = group.add(PathElement())

    d = path_node.pop('d')
    if d is None:
        raise AssertionError("Cannot split stroke and fill of non-path element")

    nodetypes = path_node.pop('sodipodi:nodetypes', None)
    path_id = path_node.pop('id', str(id(path_node)))
    transform = path_node.pop('transform', None)
    path_node.pop('style')

    # Pass along all remaining attributes to the group
    for attrib_name, attrib_value in path_node.attrib.items():
        group.set(attrib_name, attrib_value)

    group.set("id", path_id)

    # Next split apart the style attribute
    style_group = {}
    style_fill = {"stroke": "none", "fill": "#000000"}
    style_stroke = {"fill": "none", "stroke": "none"}

    for key in style.keys():
        if key.startswith("fill"):
            style_fill[key] = style[key]
        elif key.startswith("stroke"):
            style_stroke[key] = style[key]
        elif key.startswith("marker"):
            style_stroke[key] = style[key]
        elif key.startswith("filter"):
            style_group[key] = style[key]
        else:
            style_fill[key] = style[key]
            style_stroke[key] = style[key]

    if len(style_group) != 0:
        group.set("style", str(inkex.Style(style_group)))

    fill.set("style", str(inkex.Style(style_fill)))
    stroke.set("style", str(inkex.Style(style_stroke)))

    # Finalize the two paths
    fill.set("d", d)
    stroke.set("d", d)
    if nodetypes is not None:
        fill.set('sodipodi:nodetypes', nodetypes)
        stroke.set('sodipodi:nodetypes', nodetypes)
    fill.set("id", path_id + "-fill")
    stroke.set("id", path_id + "-stroke")
    if transform is not None:
        fill.set("transform", transform)
        stroke.set("transform", transform)

    # Replace the original node with the group
    path_node.getparent().replace(path_node, group)

    return [fill, stroke]


### Object related

def propagate_attribs(node, parent_style={}, parent_transform=[[1.0, 0.0, 0.0], [0.0, 1.0, 0.0]]):
    """Propagate style and transform to remove inheritance"""

    # Don't enter non-graphical portions of the document
    if not isinstance(node, (ShapeElement, SvgDocumentElement)):
        return

    # Compose the transformations
    if isinstance(node, SvgDocumentElement) and node.get("viewBox"):
        vx, vy, vw, vh = [get_dimension(x) for x in node.get_viewbox()]
        dw = get_dimension(node.get("width", vw))
        dh = get_dimension(node.get("height", vh))
        this_transform = Transform(translate=(-vx, -vy), scale=(dw / vw, dh / vh))
        del node.attrib["viewBox"]
    else:
        this_transform = Transform(parent_transform)

    this_transform *= node.transform

    # Compose the style attribs
    this_style = dict(inkex.Style.parse_str(node.get("style", "")))
    remaining_style = {}  # Style attributes that are not propagated

    non_propagated = ["filter"]  # Filters should remain on the topmost ancestor
    for key in non_propagated:
        if key in this_style.keys():
            remaining_style[key] = this_style[key]
            del this_style[key]

    # Create a copy of the parent style, and merge this style into it
    parent_style_copy = parent_style.copy()
    parent_style_copy.update(this_style)
    this_style = parent_style_copy

    # Merge in any attributes outside of the style
    style_attribs = ["fill", "stroke"]
    for attrib in style_attribs:
        if node.get(attrib):
            this_style[attrib] = node.get(attrib)
            del node.attrib[attrib]

    if isinstance(node, (SvgDocumentElement, Group, Anchor, Switch)):
        # Leave only non-propagating style attributes
        if remaining_style:
            node.style = remaining_style
        else:
            if "style" in node.keys():
                del node.attrib["style"]

        # Remove the transform attribute
        if "transform" in node.keys():
            del node.attrib["transform"]

        # Continue propagating on subelements
        for child in node.iterchildren():
            propagate_attribs(child, this_style, this_transform)
    else:
        # This element is not a container

        # Merge remaining_style into this_style
        this_style.update(remaining_style)

        # Set the element's style and transform attribs
        node.style = this_style
        node.transform = this_transform


### Style related

def get_dimension(s="1024"):
    """Convert an SVG length string from arbitrary units to pixels"""
    return inkex.units.convert_unit(s, "px")


###### Main Class #########################################
class SynfigPrep(inkex.EffectExtension):
    def effect(self):
        """Transform document in preparation for exporting it into the Synfig format"""

        a = SynfigExportActionGroup(self.document)
        self.document = a.run_document()

        # Remove inheritance of attributes
        propagate_attribs(self.document.getroot())

        # Fuse multiple subpaths in fills
        for node in self.document.getroot().xpath('//svg:path'):
            if node.get("d", "").lower().count("m") > 1:
                # There are multiple subpaths
                fill = split_fill_and_stroke(node)[0]
                if fill is not None:
                    fuse_subpaths(fill)


if __name__ == '__main__':
    SynfigPrep().run()



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2013 Nicolas Dufour (jazzynico)
# Direction code from the Restack extension, by Rob Antonishen
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.
#
"""
Merge text blocks together.
"""

import inkex
from inkex.utils import KeyDict
from inkex import (
    Rectangle, FlowRoot, FlowPara, FlowRegion, TextElement, Tspan
)

# Old settings, supported because users click 'ok' without looking.
XAN = KeyDict({'l': 'left', 'r': 'right', 'm': 'center_x'})
YAN = KeyDict({'t': 'top', 'b': 'bottom', 'm': 'center_y'})

class Merge(inkex.EffectExtension):
    """Merge text blocks together"""
    def add_arguments(self, pars):
        pars.add_argument("-d", "--direction", default="tb", help="direction to merge text")
        pars.add_argument("-x", "--xanchor", default="center_x", help="horiz point to compare")
        pars.add_argument("-y", "--yanchor", default="center_y", help="vertical point to compare")
        pars.add_argument("-k", "--keepstyle", type=inkex.Boolean, help="keep format")
        pars.add_argument("-t", "--flowtext", type=inkex.Boolean,\
            help="use a flow text structure instead of a normal text element")

    def effect(self):
        if not self.svg.selected:
            for node in self.svg.xpath('//svg:text | //svg:flowRoot'):
                self.svg.selected[node.get('id')] = node

        if not self.svg.selected:
            return

        parentnode = self.svg.get_current_layer()

        if self.options.flowtext:
            text_element = FlowRoot
            text_span = FlowPara
        else:
            text_element = TextElement
            text_span = Tspan

        text_root = parentnode.add(text_element())
        text_root.set('xml:space', 'preserve')
        text_root.style = {
            'font-size': '20px',
            'font-style': 'normal',
            'font-weight': 'normal',
            'line-height': '125%',
            'letter-spacing': '0px',
            'word-spacing': '0px',
            'fill': '#000000',
            'fill-opacity': 1,
            'stroke': 'none'
        }

        for node in sorted(self.svg.selected.values(), key=self._sort):
            self.recurse(text_span, node, text_root)

        if self.options.flowtext:
            region = text_root.add(FlowRegion())
            region.set('xml:space', 'preserve')
            rect = region.add(Rectangle())
            rect.set('xml:space', 'preserve')
            rect.set('height', 200)
            rect.set('width', 200)

    def _sort(self, node):
        return node.bounding_box().get_anchor(
            self.options.xanchor, self.options.yanchor, self.options.direction)

    def recurse(self, text_span, node, span):
        """Recursively go through each node self calling on child nodes"""
        if not isinstance(node, FlowRegion):

            newspan = span.add(text_span())
            newspan.set('xml:space', 'preserve')

            newspan.set('sodipodi:role', node.get('sodipodi:role'))
            if isinstance(node, (TextElement, FlowPara)):
                newspan.set('sodipodi:role', 'line')

            if self.options.keepstyle:
                newspan.style = node.style

            if node.text is not None:
                newspan.text = node.text
            for child in node:
                self.recurse(text_span, child, newspan)
            if node.tail and not isinstance(node, TextElement):
                newspan.tail = node.tail


if __name__ == '__main__':
    Merge().run()



#!/usr/bin/env python
# coding=utf-8
#  nicechart.py
#
#  Copyright 2011-2016
#
#  Christoph Sterz
#  Florian Weber
#  Maren Hachmann
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#
# pylint: disable=attribute-defined-outside-init

# TODO / Ideas:
# allow negative values for bar charts
# show values for stacked bar charts
# don't create a new layer for each chart, but a normal group
# correct bar height for stacked bars (it's only half as high as it should be, double)
# adjust position of heading
# use aliasing workaround for stacked bars (e.g. let the rectangles overlap)

# The extension creates one chart for a single value column in one go,
# e.g. chart all temperatures for all months of the year 1978 into one chart.
# (for this, select column 0 for labels and column 1 for values).
# "1978" etc. can be used as heading (Need not be numeric. If not used delete the heading line.)
# Month names can be used as labels
# Values can be shown, in addition to labels (doesn't work with stacked bar charts)
# Values can contain commas as decimal separator, as long as delimiter isn't comma
# Negative values are not yet supported.

# See tests/data/nicechart_01.csv for example data

import re
import csv
import math

from argparse import ArgumentTypeError

import inkex
from inkex.utils import filename_arg
from inkex import Filter, TextElement, Circle, Rectangle
from inkex.paths import Move, line

# www.sapdesignguild.org/goodies/diagram_guidelines/color_palettes.html#mss
COLOUR_TABLE = {
    "red": ["#460101", "#980101", "#d40000", "#f44800", "#fb8b00", "#eec73e", "#d9bb7a", "#fdd99b"],
    "blue": ["#000442", "#0F1781", "#252FB7", "#3A45E1", "#656DDE", "#8A91EC"],
    "gray": ["#222222", "#444444", "#666666", "#888888", "#aaaaaa", "#cccccc", "#eeeeee"],
    "contrast": ["#0000FF", "#FF0000", "#00FF00", "#CF9100", "#FF00FF", "#00FFFF"],
    "sap": ["#f8d753", "#5c9746", "#3e75a7", "#7a653e", "#e1662a", "#74796f", "#c4384f",
            "#fff8a3", "#a9cc8f", "#b2c8d9", "#bea37a", "#f3aa79", "#b5b5a9", "#e6a5a5"]
}

class NiceChart(inkex.GenerateExtension):
    """
    Inkscape extension that can draw pie charts and bar charts
    (stacked, single, horizontally or vertically)
    with optional drop shadow, from a csv file or from pasted text
    """
    container_layer = True

    @property
    def container_label(self):
        """Layer title/label"""
        return 'Chart-Layer: {}'.format(self.options.what)

    def add_arguments(self, pars):
        pars.add_argument('--tab')
        pars.add_argument('--encoding', default='utf-8')
        pars.add_argument('-w', '--what', default='22,11,67', help='Chart Values')
        pars.add_argument("-t", "--type", type=self.arg_method('render'),
                          default=self.render_bar, help="Chart Type")
        pars.add_argument("-b", "--blur", type=inkex.Boolean, default=True, help="Blur Type")
        pars.add_argument("-f", "--filename", type=filename_arg, help="Name of File")
        pars.add_argument("-i", "--input_type", default='file', help="Chart Type")
        pars.add_argument("-d", "--delimiter", default=';', help="delimiter")
        pars.add_argument("-c", "--colors", default='default', help="color-scheme")
        pars.add_argument("--colors_override", help="color-scheme-override")
        pars.add_argument("--reverse_colors", type=inkex.Boolean, default=False,
                          help="reverse color-scheme")
        pars.add_argument("-k", "--col_key", type=int, default=0,
                          help="column that contains the keys")
        pars.add_argument("-v", "--col_val", type=int, default=1,
                          help="column that contains the values")
        pars.add_argument("--headings", type=inkex.Boolean, default=True,
                          help="first line of the CSV file consists of headings for the columns")
        pars.add_argument("-r", "--rotate", type=inkex.Boolean, default=False,
                          help="Draw barchart horizontally")
        pars.add_argument("-W", "--bar-width", type=int, default=10, help="width of bars")
        pars.add_argument("-p", "--pie-radius", type=int, default=100, help="radius of pie-charts")
        pars.add_argument("-H", "--bar-height", type=int, default=100, help="height of bars")
        pars.add_argument("-O", "--bar-offset", type=int, default=5, help="distance between bars")
        pars.add_argument("--stroke-width", type=float, default=1.0)
        pars.add_argument("-o", "--text-offset", type=int, default=5,
                          help="distance between bar and descriptions")
        pars.add_argument("--heading-offset", type=int, default=50,
                          help="distance between chart and chart title")
        pars.add_argument("--segment-overlap", type=inkex.Boolean, default=False,
                          help="Remove aliasing effects by letting pie chart segments overlap")
        pars.add_argument("-F", "--font", default='sans-serif', help="font of description")
        pars.add_argument("-S", "--font-size", type=int, default=10,
                          help="font size of description")
        pars.add_argument("-C", "--font-color", default='black', help="font color of description")

        pars.add_argument("-V", "--show_values", type=inkex.Boolean, default=False,
                          help="Show values in chart")

    def get_data(self):
        """Process the data"""
        col_key = self.options.col_key
        col_val = self.options.col_val

        def process_value(val):
            """Confirm the values from files or direct"""
            val = float(val)
            if val < 0:
                raise inkex.AbortExtension("Negative values are currently not supported!")
            return val

        if self.options.input_type == "file":
            if self.options.filename is None:
                raise inkex.AbortExtension("Filename not specified!")

            # Future: use encoding when opening the file here (if ever needed)
            with open(self.options.filename, "r") as fhl:
                reader = csv.reader(fhl, delimiter=self.options.delimiter)
                title = col_val

                if self.options.headings:
                    header = next(reader)
                    title = header[col_val]

                values = [(line[col_key], process_value(line[col_val])) for line in reader]
                return (title,) + tuple(zip(*values))

        elif self.options.input_type == "direct_input":
            (keys, values) = zip(*[l.split(':', 1) for l in self.options.what.split(',')])
            return ('Direct Input', keys, [process_value(val) for val in values])

        raise inkex.AbortExtension("Unknown input type")

    def get_blur(self):
        """Add blur to the svg and return if needed"""
        if self.options.blur:
            defs = self.svg.defs
            # Create new Filter
            filt = defs.add(Filter(height='3', width='3', x='-0.5', y='-0.5'))
            # Append Gaussian Blur to that Filter
            filt.add_primitive('feGaussianBlur', stdDeviation='1.1')
            return inkex.Style(filter=filt.get_id(as_url=2))
        return inkex.Style()

    def get_color(self):
        """Get the next available color"""
        if not hasattr(self, '_colors'):
            # Generate list of available colours
            if self.options.colors_override:
                colors = self.options.colors_override.strip()
            else:
                colors = self.options.colors

            if colors[0].isalpha():
                colors = COLOUR_TABLE.get(colors.lower(), COLOUR_TABLE['red'])

            else:
                colors = re.findall("(#[0-9a-fA-F]{6})", colors)
                # to be sure we create a fallback:
                if not colors:
                    colors = COLOUR_TABLE['red']

            if self.options.reverse_colors:
                colors.reverse()
            # Cache the list of colours for later use
            self._colors = colors
            self._color_index = 0

        color = self._colors[self._color_index]
        # Increase index to the next available color
        self._color_index = (self._color_index + 1) % len(self._colors)
        return color

    def generate(self):
        """Generates a nice looking chart into SVG document."""

        # Process the data from a file or text box
        (self.title, keys, values) = self.get_data()
        if not values:
            raise inkex.AbortExtension("No data to render into a chart.")

        # Get the page attributes:
        self.width = self.svg.unittouu(self.svg.get('width'))
        self.height = self.svg.unittouu(self.svg.attrib['height'])
        self.fontoff = float(self.options.font_size) / 3

        # Check if a drop shadow should be drawn:
        self.blur = self.get_blur()

        # Draw the right type of chart
        for elem in self.options.type(keys, values):
            yield elem

    def draw_header(self, heading_x):
        """Draw an optional header text"""
        if self.options.headings and self.title:
            headingtext = self.draw_text(self.title, 4, anchor='end')
            headingtext.set("y", str(self.height / 2 + self.options.heading_offset))
            headingtext.set("x", str(heading_x))
            return headingtext
        return None

    def render_bar(self, keys, values):
        """Draw bar chart"""
        bar_height = self.options.bar_height
        bar_width = self.options.bar_width
        bar_offset = self.options.bar_offset

        # Normalize the bars to the largest value
        value_max = max(list(values) + [0.0])

        # Draw Single bars with their shadows
        for cnt, value in enumerate(values):
            # Draw each bar a set amount offset
            offset = cnt * (bar_width + bar_offset)
            bar_value = (value / value_max) * bar_height

            # Calculate the location of the bar
            x = self.width / 2 + offset
            y = self.height / 2 - int(bar_value)
            width = bar_width
            height = int(bar_value)

            if self.options.rotate:
                # Rotate the bar and align to the left
                x, y, width, height = y, x, height, width
                x += width

            for elem in self.draw_rectangle(x, y, width, height):
                yield elem

            # If keys are given, create text elements
            if keys:
                text = self.draw_text(keys[cnt], anchor='end')
                if not self.options.rotate:  # =vertical
                    text.set("transform", "rotate(-90)")
                    # y after rotation:
                    text.set("x", "-" + str(self.height / 2 + self.options.text_offset))
                    # x after rotation:
                    text.set("y", str(self.width / 2 + offset + bar_width / 2 + self.fontoff))
                else:  # =horizontal
                    text.set("y", str(self.width / 2 + offset + bar_width / 2 + self.fontoff))
                    text.set("x", str(self.height / 2 - self.options.text_offset))

                yield text

            if self.options.show_values:
                vtext = self.draw_text(int(value))
                if not self.options.rotate:  # =vertical
                    vtext.set("transform", "rotate(-90)")
                    # y after rotation:
                    vtext.set("x", "-" + str(self.height / 2 + value - self.options.text_offset))
                    # x after rotation:
                    vtext.set("y", str(self.width / 2 + offset + bar_width / 2 + self.fontoff))
                else:  # =horizontal
                    vtext.set("y", str(self.width / 2 + offset + bar_width / 2 + self.fontoff))
                    vtext.set("x", str(self.height / 2 + value + self.options.text_offset))
                yield vtext

        yield self.draw_header(self.width / 2)

    def draw_rectangle(self, x, y, width, height):
        """Draw a rectangle bar with optional shadow"""
        if self.blur:
            shadow = Rectangle(x=str(x+1), y=str(y+1), width=str(width), height=str(height))
            shadow.style = self.blur
            yield shadow

        rect = Rectangle(x=str(x), y=str(y), width=str(width), height=str(height))
        rect.set("style", "fill:" + self.get_color())
        yield rect

    def draw_text(self, text, add_size=0, anchor='start', **kwargs):
        """Draw a textual label"""
        vtext = TextElement(**kwargs)
        vtext.style = {
            'fill': self.options.font_color,
            'font-family': self.options.font,
            'font-size': str(self.options.font_size + add_size) + 'px',
            'font-style': 'normal',
            'font-variant': 'normal',
            'font-weight': 'normal',
            'font-stretch': 'normal',
            '-inkscape-font-specification': 'Bitstream Charter',
            'text-align': anchor,
            'text-anchor': anchor,
        }
        vtext.text = str(text)
        return vtext

    def render_pie_abs(self, keys, values):
        """Draw a pie chart, with absolute values"""
        # pie_abs = True
        for elem in self.render_pie(keys, values, True):
            yield elem

    def render_pie(self, keys, values, pie_abs=False):
        """Draw pie chart"""
        pie_radius = self.options.pie_radius

        # Iterate all values to draw the different slices
        color = 0
        x = float(self.width) / 2
        y = float(self.height) / 2

        # Create the shadow first (if it should be created):
        if self.blur:
            shadow = Circle(cx=str(x), cy=str(y))
            shadow.set('r', str(pie_radius))
            shadow.style = self.blur + inkex.Style(fill='#000000')
            yield shadow

        # Add a grey background circle with a light stroke
        background = Circle(cx=str(x), cy=str(y))
        background.set("r", str(pie_radius))
        background.set("style", "stroke:#ececec;fill:#f9f9f9")
        yield background

        # create value sum in order to divide the slices
        try:
            valuesum = sum(values)
        except ValueError:
            valuesum = 0

        if pie_abs:
            valuesum = 100

        # Set an offsetangle
        offset = 0

        # Draw single slices
        for cnt, value in enumerate(values):
            # Calculate the PI-angles for start and end
            angle = (2 * 3.141592) / valuesum * float(value)
            start = offset
            end = offset + angle

            # proper overlapping
            if self.options.segment_overlap:
                if cnt != len(values) - 1:
                    end += 0.09  # add a 5° overlap
                if cnt == 0:
                    start -= 0.09  # let the first element overlap into the other direction

            # then add the slice
            pieslice = inkex.PathElement()
            pieslice.set('sodipodi:type', 'arc')
            pieslice.set('sodipodi:cx', x)
            pieslice.set('sodipodi:cy', y)
            pieslice.set('sodipodi:rx', pie_radius)
            pieslice.set('sodipodi:ry', pie_radius)
            pieslice.set('sodipodi:start', start)
            pieslice.set('sodipodi:end', end)
            pieslice.set("style", "fill:" + self.get_color() + ";stroke:none;fill-opacity:1")
            ang = angle / 2 + offset

            # If text is given, draw short paths and add the text
            if keys:
                elem = inkex.PathElement()
                elem.path = [
                    Move(
                        (self.width / 2) + pie_radius * math.cos(ang),
                        (self.height / 2) + pie_radius * math.sin(ang),
                    ), line(
                        (self.options.text_offset - 2) * math.cos(ang),
                        (self.options.text_offset - 2) * math.sin(ang),
                    ),
                ]

                elem.style = {
                    'fill': 'none',
                    'stroke': self.options.font_color,
                    'stroke-width': self.options.stroke_width,
                    'stroke-linecap': 'butt',
                }
                yield elem

                label = keys[cnt]
                if self.options.show_values:
                    label += ' ({}{})'.format(str(value), ('', '%')[pie_abs])

                # check if it is right or left of the Pie
                anchor = 'start' if math.cos(ang) > 0 else 'end'
                text = self.draw_text(label, anchor=anchor)

                off = pie_radius + self.options.text_offset
                text.set("x", (self.width / 2) + off * math.cos(ang))
                text.set("y", (self.height / 2) + off * math.sin(ang) + self.fontoff)
                yield text

            # increase the rotation-offset and the colorcycle-position
            offset = offset + angle
            color = (color + 1) % 8

            # append the objects to the extension-layer
            yield pieslice

        yield self.draw_header(self.width / 2 - pie_radius)

    def render_stbar(self, keys, values):
        """Draw stacked bar chart"""

        # Iterate over all values to draw the different slices
        color = 0

        # create value sum in order to divide the bars
        try:
            valuesum = sum(values)
        except ValueError:
            valuesum = 0.0

        # Init offset
        offset = 0
        width = self.options.bar_width
        height = self.options.bar_height
        x = self.width / 2
        y = self.height / 2

        if self.blur:
            if self.options.rotate:
                width, height = height, width
                shy = y
            else:
                shy = str(y - self.options.bar_height)

            # Create rectangle element
            shadow = Rectangle(
                x=str(x), y=str(shy),
                width=str(width), height=str(height),
            )

            # Set shadow blur (connect to filter object in xml path)
            shadow.style = self.blur
            yield shadow

        # Draw Single bars
        for cnt, value in enumerate(values):

            # Calculate the individual heights normalized on 100units
            normedvalue = (self.options.bar_height / valuesum) * float(value)

            # Create rectangle element
            rect = Rectangle()

            # Set chart position to center of document.
            if not self.options.rotate:
                rect.set('x', str(self.width / 2))
                rect.set('y', str(self.height / 2 - offset - normedvalue))
                rect.set("width", str(self.options.bar_width))
                rect.set("height", str(normedvalue))
            else:
                rect.set('x', str(self.width / 2 + offset))
                rect.set('y', str(self.height / 2))
                rect.set("height", str(self.options.bar_width))
                rect.set("width", str(normedvalue))

            rect.set("style", "fill:" + self.get_color())

            # If text is given, draw short paths and add the text
            # TODO: apply overlap workaround for visible gaps in between
            if keys:
                if not self.options.rotate:
                    x1 = (self.width + self.options.bar_width) / 2
                    y1 = y - offset - (normedvalue / 2)
                    x2 = self.options.bar_width / 2 + self.options.text_offset
                    y2 = 0
                    txt = self.width / 2 + self.options.bar_width + self.options.text_offset + 1
                    tyt = y - offset + self.fontoff - (normedvalue / 2)
                else:
                    x1 = x + offset + normedvalue / 2
                    y1 = y + self.options.bar_width / 2
                    x2 = 0
                    y2 = self.options.bar_width / 2 + (self.options.font_size \
                            * cnt) + self.options.text_offset
                    txt = x + offset + normedvalue / 2 - self.fontoff
                    tyt = (y) + self.options.bar_width + (self.options.font_size \
                            * (cnt + 1)) + self.options.text_offset

                elem = inkex.PathElement()
                elem.path = [Move(x1, y1), line(x2, y2)]
                elem.style = {
                    'fill': 'none',
                    'stroke': self.options.font_color,
                    'stroke-width': self.options.stroke_width,
                    'stroke-linecap': 'butt',
                }
                yield elem
                yield self.draw_text(keys[cnt], x=str(txt), y=str(tyt))

            # Increase Offset and Color
            offset = offset + normedvalue
            color = (color + 1) % 8

            # Draw rectangle
            yield rect

        yield self.draw_header(self.width / 2 + offset + normedvalue)


if __name__ == '__main__':
    NiceChart().run()



#!/usr/bin/env python
"""Increase lightness"""

import inkex

class MoreLight(inkex.ColorExtension):
    """Lighten selected objects"""
    def modify_color(self, name, color):
        color.lightness += int(0.05 * 255.0)
        return color

if __name__ == '__main__':
    MoreLight().run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2020 Martin Owens <doctormo@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>
#
"""
Bootstrap the inkscape extensions manager setting up the virtualenv
and adding the extensions package.
"""

import os
import io
import sys

import inkex
from inkex.utils import get_user_directory
from inkex.base import InkscapeExtension
from inkex.command import CommandNotFound, ProgramRunError, call

TARGET_DIR = get_user_directory()
FALLBACK_DIR = os.path.join(TARGET_DIR or './', 'org.inkscape.inkman')
if os.path.isdir(FALLBACK_DIR):
    sys.path.insert(0, FALLBACK_DIR)

try:
    from manage_extensions import run as run_existing
except ImportError:
    run_existing = None

FALLBACK_URL = "https://media.inkscape.org/static/extensions-manager-fallback.zip"

class Bootstrap(InkscapeExtension):
    multi_inx = True

    def add_arguments(self, pars):
        pars.add_argument('--tab')
        pars.add_argument('--version', default='inkscape-extensions-manager')

    def load_raw(self):
        pass

    def save_raw(self, ret):
        if os.path.isdir(FALLBACK_DIR):
            sys.path.insert(0, FALLBACK_DIR)
        elif TARGET_DIR:
            sys.path.insert(0, TARGET_DIR)
        try:
            from manage_extensions import run as run_existing
        except (ImportError, ModuleNotFoundError):
            return self.msg("Extension manager installed, please re-run.")
        run_existing(sys.argv)

    def effect(self):
        fallback = False
        try:
            call('virtualenv', TARGET_DIR, p='python3')
        except CommandNotFound:
            fallback = True
        except ProgramRunError as err:
            raise inkex.AbortExtension(
                "There has been a problem creating the python environment:\n" + str(err))

        if fallback:
            # Add a fallback for places like windows where python isn't available.
            if self.install_fallback():
                return
            raise inkex.AbortExtension(
                "You must have the python-virtualenv package installed. This should have"
                " been included with Inkscape, but in some special cases it might not"
                " be. Please install this software externally and try again.")

        try:
            call(os.path.join(TARGET_DIR, 'bin', 'pip'), 'install', self.options.version)
        except CommandNotFound:
            raise inkex.AbortExtension("Can't find pip program after environment initialisation!")
        except ProgramRunError as err:
            raise inkex.AbortExtension("Error installing extension manager package:\n" + str(err))

    def install_fallback(self):
        """
        A pre-set zip installer which has been prepared for installation.
        """
        from urllib3.exceptions import NewConnectionError
        import requests
        import zipfile

        # Get remove content
        done = False
        session = requests.session()
        try:
            remote = session.get(FALLBACK_URL)
            if remote and remote.status_code == 200:
                with zipfile.ZipFile(io.BytesIO(remote.content)) as archive:
                    for filename in archive.namelist():
                        if '.inx' in filename:
                            done = True
                        self._install_file(archive.read(filename),
                            os.path.join(FALLBACK_DIR, filename))
        except NewConnectionError:
            self.msg("Could not connect to the internet, please check connection and try again!")
        finally:
            session.close()
        return done

    def _install_file(self, content, filename):
        """Install the content into the given location"""
        filedir = os.path.dirname(filename)
        if not os.path.isdir(filedir):
            os.makedirs(filedir)
        if not os.path.isdir(filename):
            with open(filename, 'wb') as fhl:
                fhl.write(content)

if __name__ == '__main__':
    if run_existing is not None:
        # If the extension manager is already installed
        # Run it instead of the bootstrap process.
        run_existing(sys.argv)
    else:
        Bootstrap().run()



# coding=utf-8
#
# Copyright (C) 2010 Geoffrey Mosini
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
#
"""
Generate barcodes for Code25-interleaved 2 of 5, for Inkscape.
"""

from .Base import Barcode

# 1 means thick, 0 means thin
ENCODE = {
    '0': '00110',
    '1': '10001',
    '2': '01001',
    '3': '11000',
    '4': '00101',
    '5': '10100',
    '6': '01100',
    '7': '00011',
    '8': '10010',
    '9': '01010',
}


class Code25i(Barcode):
    """Convert a text into string binary of black and white markers"""

    # Start and stop code are already encoded into white (0) and black(1) bars
    def encode(self, number):
        if not number.isdigit():
            return self.error(number, "CODE25 can only encode numbers.")

        # Number of figures to encode must be even,
        # a 0 is added to the left in case it's odd.
        if len(number) % 2 > 0:
            number = '0' + number

        # Number is encoded by pairs of 2 figures
        size = len(number) // 2
        encoded = '1010'
        for i in range(size):
            # First in the pair is encoded in black (1), second in white (0)
            black = ENCODE[number[i * 2]]
            white = ENCODE[number[i * 2 + 1]]
            for j in range(5):
                if black[j] == '1':
                    encoded += '11'
                else:
                    encoded += '1'
                if white[j] == '1':
                    encoded += '00'
                else:
                    encoded += '0'
        return encoded + '1101'



#!/usr/bin/env python
"""Replace color extension"""

import inkex

class ReplaceColor(inkex.ColorExtension):
    """Replace color in SVG with another"""
    def add_arguments(self, pars):
        pars.add_argument("--tab")
        pars.add_argument('-f', "--from_color",\
            default=inkex.Color("black"), type=inkex.Color, help="Replace color")
        pars.add_argument('-t', "--to_color",\
            default=inkex.Color("red"), type=inkex.Color, help="By color")

    def modify_color(self, name, color):
        return self.options.to_color if color == self.options.from_color else color

if __name__ == '__main__':
    ReplaceColor().run()



#!/usr/bin/env python
# coding=utf-8
#
# Authors:
#   Nicolas Dufour - Association Inkscape-fr
#   Aurelio A. Heckert <aurium(a)gmail.com>
#
# Copyright (C) 2008 Authors
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
This extension allows you to draw crop, registration and other
printing marks in Inkscape.
"""

import math
import inkex
from inkex import Circle, Rectangle, TextElement


class PrintingMarks(inkex.EffectExtension):
    # Default parameters
    stroke_width = 0.25

    def add_arguments(self, pars):
        pars.add_argument("--where", help="Apply crop marks to...", default="canvas")
        pars.add_argument("--crop_marks", type=inkex.Boolean, default=True, help="Draw crop Marks")
        pars.add_argument("--bleed_marks", type=inkex.Boolean, help="Draw Bleed Marks")
        pars.add_argument("--registration_marks", type=inkex.Boolean,\
            dest="reg_marks", default=True, help="Draw Registration Marks?")
        pars.add_argument("--star_target", type=inkex.Boolean, help="Draw Star Target?")
        pars.add_argument("--colour_bars", type=inkex.Boolean, help="Draw Colour Bars?", default=True)
        pars.add_argument("--page_info", type=inkex.Boolean, help="Draw Page Information?")
        pars.add_argument("--unit", default="px", help="Draw measurement")
        pars.add_argument("--crop_offset", type=float, default=5.0, help="Offset")
        pars.add_argument("--bleed_top", type=float, default=5.0, help="Bleed Top Size")
        pars.add_argument("--bleed_bottom", type=float, default=5.0, help="Bleed Bottom Size")
        pars.add_argument("--bleed_left", type=float, default=5.0, help="Bleed Left Size")
        pars.add_argument("--bleed_right", type=float, default=5.0, help="Bleed Right Size")
        pars.add_argument("--tab", help="The selected UI-tab when OK was pressed")

    def draw_crop_line(self, x1, y1, x2, y2, name, parent):
        style = {'stroke': '#000000', 'stroke-width': str(self.stroke_width),
                 'fill': 'none'}
        line_attribs = {'style': str(inkex.Style(style)),
                        'id': name,
                        'd': 'M ' + str(x1) + ',' + str(y1) + ' L ' + str(x2) + ',' + str(y2)}
        parent.add(inkex.PathElement(**line_attribs))

    def draw_bleed_line(self, x1, y1, x2, y2, name, parent):
        style = {'stroke': '#000000', 'stroke-width': str(self.stroke_width),
                 'fill': 'none',
                 'stroke-miterlimit': '4', 'stroke-dasharray': '4, 2, 1, 2',
                 'stroke-dashoffset': '0'}
        line_attribs = {'style': str(inkex.Style(style)),
                        'id': name,
                        'd': 'M ' + str(x1) + ',' + str(y1) + ' L ' + str(x2) + ',' + str(y2)}
        parent.add(inkex.PathElement(**line_attribs))

    def draw_reg_circles(self, cx, cy, r, name, colours, parent):
        for i in range(len(colours)):
            style = {'stroke': colours[i], 'stroke-width': str(r / len(colours)),
                     'fill': 'none'}
            circle_attribs = {'style': str(inkex.Style(style)),
                              'inkscape:label': name,
                              'cx': str(cx), 'cy': str(cy),
                              'r': str((r / len(colours)) * (i + 0.5))}
            parent.add(Circle(**circle_attribs))

    def draw_reg_marks(self, cx, cy, rotate, name, parent):
        colours = ['#000000', '#00ffff', '#ff00ff', '#ffff00', '#000000']
        g = parent.add(inkex.Group(id=name))
        for i in range(len(colours)):
            style = {'fill': colours[i], 'fill-opacity': '1', 'stroke': 'none'}
            r = (self.mark_size / 2)
            step = r
            stroke = r / len(colours)
            regoffset = stroke * i
            regmark_attribs = {'style': str(inkex.Style(style)),
                               'd': 'm' +
                                    ' ' + str(-regoffset) + ',' + str(r) +
                                    ' ' + str(-stroke) + ',0' +
                                    ' ' + str(step) + ',' + str(-r) +
                                    ' ' + str(-step) + ',' + str(-r) +
                                    ' ' + str(stroke) + ',0' +
                                    ' ' + str(step) + ',' + str(r) +
                                    ' ' + str(-step) + ',' + str(r) +
                                    ' z',
                               'transform': 'translate(' + str(cx) + ',' + str(cy) +
                                            ') rotate(' + str(rotate) + ')'}
            g.add(inkex.PathElement(**regmark_attribs))

    def draw_star_target(self, cx, cy, name, parent):
        r = (self.mark_size / 2)
        style = {'fill': '#000 device-cmyk(1,1,1,1)', 'fill-opacity': '1', 'stroke': 'none'}
        d = ' M 0,0'
        i = 0
        while i < (2 * math.pi):
            i += math.pi / 16
            d += ' L 0,0 ' + \
                 ' L ' + str(math.sin(i) * r) + ',' + str(math.cos(i) * r) + \
                 ' L ' + str(math.sin(i + 0.09) * r) + ',' + str(math.cos(i + 0.09) * r)
        elem = parent.add(inkex.PathElement())
        elem.label = name
        elem.transform.add_translate(cx, cy)
        elem.path = d
        elem.style = inkex.Style(style)

    def draw_coluor_bars(self, cx, cy, rotate, name, parent, bbox):
        group = parent.add(inkex.Group(id=name))
        group.transform = inkex.Transform(translate=(cx, cy)) * inkex.Transform(rotate=rotate)
        loc = 0
        if bbox:
            loc = min(self.mark_size / 3, max(bbox.width, bbox.height) / 45)
        for bar in [{'c': '*', 'stroke': '#000', 'x': 0, 'y': -(loc + 1)},
                    {'c': 'r', 'stroke': '#0FF', 'x': 0, 'y': 0},
                    {'c': 'g', 'stroke': '#F0F', 'x': (loc * 11) + 1, 'y': -(loc + 1)},
                    {'c': 'b', 'stroke': '#FF0', 'x': (loc * 11) + 1, 'y': 0}
                    ]:
            i = 0
            while i <= 1:
                color = inkex.Color('white')
                if bar['c'] == 'r' or bar['c'] == '*':
                    color.red = 255 * i
                if bar['c'] == 'g' or bar['c'] == '*':
                    color.green = 255 * i
                if bar['c'] == 'b' or bar['c'] == '*':
                    color.blue = 255 * i
                r_att = {'fill': str(color),
                         'stroke': bar['stroke'],
                         'stroke-width': loc/8,
                         'x': str((loc * i * 10) + bar['x']), 'y': str(bar['y']),
                         'width': loc, 'height': loc}
                rect = Rectangle()
                for att, value in r_att.items():
                    rect.set(att, value)
                group.add(rect)
                i += 0.1

    def effect(self):
        self.mark_size = self.svg.unittouu('1cm')
        self.min_mark_margin = self.svg.unittouu('3mm')

        if self.options.where == 'selection':
            bbox = self.svg.selection.bounding_box()
            if bbox is None:
                raise inkex.AbortExtension(_("Selection is empty"))
        else:
            bbox = self.svg.get_page_bbox()

        # Get SVG document dimensions
        # self.width must be replaced by bbox.right. same to others.
        svg = self.document.getroot()

        # Convert parameters to user unit
        offset = self.svg.unittouu(str(self.options.crop_offset) +
                                   self.options.unit)
        bt = self.svg.unittouu(str(self.options.bleed_top) + self.options.unit)
        bb = self.svg.unittouu(str(self.options.bleed_bottom) + self.options.unit)
        bl = self.svg.unittouu(str(self.options.bleed_left) + self.options.unit)
        br = self.svg.unittouu(str(self.options.bleed_right) + self.options.unit)
        # Bleed margin
        if bt < offset:
            bmt = 0
        else:
            bmt = bt - offset
        if bb < offset:
            bmb = 0
        else:
            bmb = bb - offset
        if bl < offset:
            bml = 0
        else:
            bml = bl - offset
        if br < offset:
            bmr = 0
        else:
            bmr = br - offset

        # Define the new document limits
        offset_left = bbox.left - offset
        offset_right = bbox.right + offset
        offset_top = bbox.top - offset
        offset_bottom = bbox.bottom + offset

        # Get middle positions
        middle_vertical = bbox.top + (bbox.height / 2)
        middle_horizontal = bbox.left + (bbox.width / 2)

        # Test if printing-marks layer existis
        layer = self.svg.xpath('//*[@id="printing-marks" and @inkscape:groupmode="layer"]')
        if layer:
            svg.remove(layer[0])  # remove if it existis
        # Create a new layer
        layer = svg.add(inkex.Layer.new("Printing Marks"))
        layer.set('id', 'printing-marks')
        layer.set('sodipodi:insensitive', 'true')

        # Crop Mark
        if self.options.crop_marks:
            # Create a group for Crop Mark
            g_crops = layer.add(inkex.Group(id='CropMarks'))
            g_crops.label = 'CropMarks'

            # Top left Mark
            self.draw_crop_line(bbox.left, offset_top,
                                bbox.left, offset_top - self.mark_size,
                                'cropTL1', g_crops)
            self.draw_crop_line(offset_left, bbox.top,
                                offset_left - self.mark_size, bbox.top,
                                'cropTL2', g_crops)

            # Top right Mark
            self.draw_crop_line(bbox.right, offset_top,
                                bbox.right, offset_top - self.mark_size,
                                'cropTR1', g_crops)
            self.draw_crop_line(offset_right, bbox.top,
                                offset_right + self.mark_size, bbox.top,
                                'cropTR2', g_crops)

            # Bottom left Mark
            self.draw_crop_line(bbox.left, offset_bottom,
                                bbox.left, offset_bottom + self.mark_size,
                                'cropBL1', g_crops)
            self.draw_crop_line(offset_left, bbox.bottom,
                                offset_left - self.mark_size, bbox.bottom,
                                'cropBL2', g_crops)

            # Bottom right Mark
            self.draw_crop_line(bbox.right, offset_bottom,
                                bbox.right, offset_bottom + self.mark_size,
                                'cropBR1', g_crops)
            self.draw_crop_line(offset_right, bbox.bottom,
                                offset_right + self.mark_size, bbox.bottom,
                                'cropBR2', g_crops)

        # Bleed Mark
        if self.options.bleed_marks:
            # Create a group for Bleed Mark
            g_bleed = layer.add(inkex.Group())
            g_bleed.label = 'BleedMarks'
            g_bleed.set('id', 'BleedMarks')

            # Top left Mark
            self.draw_bleed_line(bbox.left - bl, offset_top - bmt,
                                 bbox.left - bl, offset_top - bmt - self.mark_size,
                                 'bleedTL1', g_bleed)
            self.draw_bleed_line(offset_left - bml, bbox.top - bt,
                                 offset_left - bml - self.mark_size, bbox.top - bt,
                                 'bleedTL2', g_bleed)

            # Top right Mark
            self.draw_bleed_line(bbox.right + br, offset_top - bmt,
                                 bbox.right + br, offset_top - bmt - self.mark_size,
                                 'bleedTR1', g_bleed)
            self.draw_bleed_line(offset_right + bmr, bbox.top - bt,
                                 offset_right + bmr + self.mark_size, bbox.top - bt,
                                 'bleedTR2', g_bleed)

            # Bottom left Mark
            self.draw_bleed_line(bbox.left - bl, offset_bottom + bmb,
                                 bbox.left - bl, offset_bottom + bmb + self.mark_size,
                                 'bleedBL1', g_bleed)
            self.draw_bleed_line(offset_left - bml, bbox.bottom + bb,
                                 offset_left - bml - self.mark_size, bbox.bottom + bb,
                                 'bleedBL2', g_bleed)

            # Bottom right Mark
            self.draw_bleed_line(bbox.right + br, offset_bottom + bmb,
                                 bbox.right + br, offset_bottom + bmb + self.mark_size,
                                 'bleedBR1', g_bleed)
            self.draw_bleed_line(offset_right + bmr, bbox.bottom + bb,
                                 offset_right + bmr + self.mark_size, bbox.bottom + bb,
                                 'bleedBR2', g_bleed)

        # Registration Mark
        if self.options.reg_marks:
            # Create a group for Registration Mark
            g_center = layer.add(inkex.Group())
            g_center.label = 'RegistrationMarks'
            g_center.set('id', 'RegistrationMarks')
            # Left Mark
            cx = max(bml + offset, self.min_mark_margin)
            self.draw_reg_marks(bbox.left - cx - (self.mark_size / 2),
                                middle_vertical - self.mark_size * 1.5,
                                '0', 'regMarkL', g_center)

            # Right Mark
            cx = max(bmr + offset, self.min_mark_margin)
            self.draw_reg_marks(bbox.right + cx + (self.mark_size / 2),
                                middle_vertical - self.mark_size * 1.5,
                                '180', 'regMarkR', g_center)

            # Top Mark
            cy = max(bmt + offset, self.min_mark_margin)
            self.draw_reg_marks(middle_horizontal,
                                bbox.top - cy - (self.mark_size / 2),
                                '90', 'regMarkT', g_center)

            # Bottom Mark
            cy = max(bmb + offset, self.min_mark_margin)
            self.draw_reg_marks(middle_horizontal,
                                bbox.bottom + cy + (self.mark_size / 2),
                                '-90', 'regMarkB', g_center)

        # Star Target
        if self.options.star_target:
            # Create a group for Star Target
            g_center = layer.add(inkex.Group())
            g_center.label = 'StarTarget'
            g_center.set('id', 'StarTarget')

            if bbox.height < bbox.width:
                # Left Star
                cx = max(bml + offset, self.min_mark_margin)
                self.draw_star_target(bbox.left - cx - (self.mark_size / 2),
                                      middle_vertical,
                                      'starTargetL', g_center)
                # Right Star
                cx = max(bmr + offset, self.min_mark_margin)
                self.draw_star_target(bbox.right + cx + (self.mark_size / 2),
                                      middle_vertical,
                                      'starTargetR', g_center)
            else:
                # Top Star
                cy = max(bmt + offset, self.min_mark_margin)
                self.draw_star_target(middle_horizontal - self.mark_size * 1.5,
                                      bbox.top - cy - (self.mark_size / 2),
                                      'starTargetT', g_center)
                # Bottom Star
                cy = max(bmb + offset, self.min_mark_margin)
                self.draw_star_target(middle_horizontal - self.mark_size * 1.5,
                                      bbox.bottom + cy + (self.mark_size / 2),
                                      'starTargetB', g_center)

        # Colour Bars
        if self.options.colour_bars:
            # Create a group for Colour Bars
            g_center = layer.add(inkex.Group())
            g_center.label = 'ColourBars'
            g_center.set('id', 'PrintingColourBars')

            if bbox.height > bbox.width:
                # Left Bars
                cx = max(bml + offset, self.min_mark_margin)
                self.draw_coluor_bars(bbox.left - cx - (self.mark_size / 2),
                                      middle_vertical + self.mark_size,
                                      90,
                                      'PrintingColourBarsL', g_center, bbox)
                # Right Bars
                cx = max(bmr + offset, self.min_mark_margin)
                self.draw_coluor_bars(bbox.right + cx + (self.mark_size / 2),
                                      middle_vertical + self.mark_size,
                                      90,
                                      'PrintingColourBarsR', g_center, bbox)
            else:
                # Top Bars
                cy = max(bmt + offset, self.min_mark_margin)
                self.draw_coluor_bars(middle_horizontal + self.mark_size,
                                      bbox.top - cy - (self.mark_size / 2),
                                      0,
                                      'PrintingColourBarsT', g_center, bbox)
                # Bottom Bars
                cy = max(bmb + offset, self.min_mark_margin)
                self.draw_coluor_bars(middle_horizontal + self.mark_size,
                                      bbox.bottom + cy + (self.mark_size / 2),
                                      0,
                                      'PrintingColourBarsB', g_center, bbox)

        # Page Information
        if self.options.page_info:
            # Create a group for Page Information
            g_pag_info = layer.add(inkex.Group())
            g_pag_info.label = 'PageInformation'
            g_pag_info.set('id', 'PageInformation')
            y_margin = max(bmb + offset, self.min_mark_margin)
            txt_attribs = {
                'style': 'font-size:12px;font-style:normal;font-weight:normal;fill:#000000;font-family:Bitstream Vera Sans,sans-serif;text-anchor:middle;text-align:center',
                'x': str(middle_horizontal),
                'y': str(bbox.bottom + y_margin + self.mark_size + 20)
            }
            txt = g_pag_info.add(TextElement(**txt_attribs))
            txt.text = 'Page size: ' + \
                       str(round(self.svg.uutounit(bbox.width, self.options.unit), 2)) + \
                       'x' + \
                       str(round(self.svg.uutounit(bbox.height, self.options.unit), 2)) + \
                       ' ' + self.options.unit


if __name__ == '__main__':
    PrintingMarks().run()



# coding=utf-8
#
# Copyright (C) 2007 Martin Owens
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Python barcode renderer for Code39 Extended barcodes. Designed for Inkscape.
"""

from .Code39 import Code39

encode = list('ABCDEFGHIJKLMNOPQRSTUVWXYZ')

map = {}

i = 0
for char in encode:
    map[char] = i
    i += 1


# Extended encoding maps for full ASCII Code93
def getMap(array):
    result = {}
    y = 0
    for x in array:
        result[chr(x)] = encode[y]
        y += 1

    return result


# MapA is eclectic, but B, C, D are all ASCII ranges
mapA = getMap([27, 28, 29, 30, 31, 59, 60, 61, 62, 63, 91, 92, 93, 94, 95, 123, 124, 125, 126, 127, 0, 64, 96, 127, 127, 127])  # %
mapB = getMap(range(1, 26))  # $
mapC = getMap(range(33, 58))  # /
mapD = getMap(range(97, 122))  # +


class Code39Ext(Code39):
    def encode(self, text):
        # We are only going to extend the Code39 barcodes
        result = ''
        for char in text:
            if char in mapA:
                char = '%' + mapA[char]
            elif char in mapB:
                char = '$' + mapB[char]
            elif char in mapC:
                char = '/' + mapC[char]
            elif char in mapD:
                char = '+' + mapD[char]
            result = result + char

        return Code39.encode(self, result)



#!/usr/bin/env python
"""
Convert PNG to Tiff using Raster Output extension.
"""

import inkex

class TiffOutput(inkex.RasterOutputExtension):
    def add_arguments(self, pars):
        pars.add_argument('--tab')
        pars.add_argument('--compression', default=None)
        pars.add_argument('--quality', type=int, default=95)

    def save(self, stream):
        self.img.convert('RGB').save(stream,
            format='tiff',
            compression=(self.options.compression or None),
            quality=100)
        # TODO: Add other fields such as copyright etc.

if __name__ == '__main__':
    TiffOutput().run()



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2013 Public Domain
#               2018 Martin Owens <doctormo@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

import sys
import hpgl_decoder
import inkex
from inkex.localization import inkex_gettext as _

class HpglInput(inkex.InputExtension):
    def add_arguments(self, pars):
        pars.add_argument('--resolutionX', type=float, default=1016.0, help='Resolution X (dpi)')
        pars.add_argument('--resolutionY', type=float, default=1016.0, help='Resolution Y (dpi)')
        pars.add_argument('--showMovements', type=inkex.Boolean, default=False,
                          help='Show Movements between paths')

    def load(self, stream):
        return b';'.join(line.strip() for line in stream).decode()

    def effect(self):
        # interpret HPGL data
        myHpglDecoder = hpgl_decoder.hpglDecoder(self.document, self.options)
        self.document = None

        try:
            doc, warnings = myHpglDecoder.get_svg()
        except Exception as inst:
            if inst.args[0] == 'NO_HPGL_DATA':
                # issue error if no hpgl data found
                inkex.errormsg(_("No HPGL data found."))
                exit(1)
            else:
                type, value, traceback = sys.exc_info()
                raise ValueError("", type, value).with_traceback(traceback)

        # issue warning if unknown commands where found
        if 'UNKNOWN_COMMANDS' in warnings:
            inkex.errormsg(_("The HPGL data contained unknown (unsupported) commands, there is a possibility that the drawing is missing some content."))

        # deliver document to inkscape
        self.document = doc

if __name__ == '__main__':
    HpglInput().run()



#!/usr/bin/env python
"""Reduce hue"""

import inkex

class LessHue(inkex.ColorExtension):
    """Remove Hue from the color"""
    def modify_color(self, name, color):
        color.hue -= int(0.05 * 255)
        return color

if __name__ == '__main__':
    LessHue().run()



# coding=utf-8
#
# Copyright (C) 2018 - Martin Owens <doctormo@mgail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Provide some documentation to existing extensions about why they're failing.
"""
#
# We ignore a lot of pylint warnings here:
#
# pylint: disable=invalid-name,unused-argument,missing-docstring,too-many-public-methods
#

import os
import sys
import traceback
import warnings
import argparse
from argparse import ArgumentParser

import inkex
import inkex.utils
import inkex.units
from inkex.base import SvgThroughMixin, InkscapeExtension
from inkex.localization import inkex_gettext as _
from inkex.elements._base import BaseElement

warnings.simplefilter("default")
# To load each of the deprecated sub-modules (the ones without a namespace)
# we will add the directory to our pythonpath so older scripts can find them

INKEX_DIR = os.path.abspath(os.path.dirname(__file__))
SIMPLE_DIR = os.path.join(INKEX_DIR, 'deprecated-simple')

if os.path.isdir(SIMPLE_DIR):
    sys.path.append(SIMPLE_DIR)

try:
    DEPRECATION_LEVEL = int(os.environ.get('INKEX_DEPRECATION_LEVEL', 1))
except ValueError:
    DEPRECATION_LEVEL = 1

def _deprecated(msg, stack=2, level=DEPRECATION_LEVEL):
    """Internal method for raising a deprecation warning"""
    if level > 1:
        msg += ' ; '.join(traceback.format_stack())
    if level:
        warnings.warn(msg, category=DeprecationWarning, stacklevel=stack + 1)

class DeprecatedEffect(object):
    """An Inkscape effect, takes SVG in and outputs SVG, providing a deprecated layer"""

    def __init__(self):
        super(DeprecatedEffect, self).__init__()

        self._doc_ids = None

        # These are things we reference in the deprecated code, they are provided
        # by the new effects code, but we want to keep this as a Mixin so these
        # items will keep pylint happy and let use check our code as we write.
        if not hasattr(self, 'svg'):
            from .elements import SvgDocumentElement
            self.svg = SvgDocumentElement()
        if not hasattr(self, 'arg_parser'):
            self.arg_parser = ArgumentParser()
        if not hasattr(self, 'run'):
            self.run = self.affect

    @classmethod
    def _deprecated(cls, name, msg=_('{} is deprecated and should be removed'), stack=3):
        """Give the user a warning about their extension using a deprecated API"""
        _deprecated(
            msg.format('Effect.' + name, cls=cls.__module__ + '.' + cls.__name__),
            stack=stack)

    @property
    def OptionParser(self):
        self._deprecated(
            'OptionParser',
            _('{} or `optparse` has been deprecated and replaced with `argparser`. '
              'You must change `self.OptionParser.add_option` to '
              '`self.arg_parser.add_argument`; the arguments are similar.'))
        return self

    def add_option(self, *args, **kw):
        # Convert type string into type method as needed
        if 'type' in kw:
            kw['type'] = {
                'string': str,
                'int': int,
                'float': float,
                'inkbool': inkex.utils.Boolean,
            }.get(kw['type'])
        if kw.get('action', None) == 'store':
            # Default store action not required, removed.
            kw.pop('action')
        args = [arg for arg in args if arg != ""]
        self.arg_parser.add_argument(*args, **kw)

    def effect(self):
        self._deprecated('effect', _('{} method is now a required method. It should '
                                     'be created on {cls}, even if it does nothing.'))

    @property
    def current_layer(self):
        self._deprecated('current_layer',\
            _('{} is now a method in the SvgDocumentElement class. Use `self.svg.get_current_layer()` instead.'))
        return self.svg.get_current_layer()

    @property
    def view_center(self):
        self._deprecated('view_center',\
            _('{} is now a method in the SvgDocumentElement class. Use `self.svg.get_center_position()` instead.'))
        return self.svg.namedview.center

    @property
    def selected(self):
        self._deprecated('selected', _('{} is now a dict in the SvgDocumentElement class. Use `self.svg.selected`.'))
        return dict([(elem.get('id'), elem) for elem in self.svg.selected])

    @property
    def doc_ids(self):
        self._deprecated('doc_ids', _('{} is now a method in the SvgDocumentElement class. '
                                      'Use `self.svg.get_ids()` instead.'))
        if self._doc_ids is None:
            self._doc_ids = dict.fromkeys(self.svg.get_ids())
        return self._doc_ids

    def getdocids(self):
        self._deprecated('getdocids', _('Use `self.svg.get_ids()` instead of {} and `doc_ids`.'))
        self._doc_ids = None
        self.svg.ids.clear()

    def getselected(self):
        self._deprecated('getselected', _('{} has been removed'))

    def getElementById(self, eid):
        self._deprecated('getElementById',\
            _('{} is now a method in the SvgDocumentElement class. Use `self.svg.getElementById(eid)` instead.'))
        return self.svg.getElementById(eid)

    def xpathSingle(self, xpath):
        self._deprecated('xpathSingle', _('{} is now a new method in the SvgDocumentElement class. '
                                          'Use `self.svg.getElement(path)` instead.'))
        return self.svg.getElement(xpath)

    def getParentNode(self, node):
        self._deprecated('getParentNode',\
            _('{} is no longer in use. Use the lxml `.getparent()` method instead.'))
        return node.getparent()

    def getNamedView(self):
        self._deprecated('getNamedView',\
            _('{} is now a property of the SvgDocumentElement class. '
              'Use `self.svg.namedview` to access this element.'))
        return self.svg.namedview

    def createGuide(self, posX, posY, angle):
        from .elements import Guide
        self._deprecated('createGuide',\
            _('{} is now a method of the namedview element object. '
              'Use `self.svg.namedview.add(Guide().move_to(x, y, a))` instead.'))
        return self.svg.namedview.add(Guide().move_to(posX, posY, angle))

    def affect(self, args=sys.argv[1:], output=True):  # pylint: disable=dangerous-default-value
        # We need a list as the default value to preserve backwards compatibility
        self._deprecated('affect', _('{} is now `Effect.run()`. The `output` argument has changed.'))
        self._args = args[-1:]
        return self.run(args=args)

    @property
    def args(self):
        self._deprecated('args', _('self.args[-1] is now self.options.input_file.'))
        return self._args

    @property
    def svg_file(self):
        self._deprecated('svg_file', _('self.svg_file is now self.options.input_file.'))
        return self.options.input_file

    def save_raw(self, ret):
        # Derived class may implement "output()"
        # Attention: 'cubify.py' implements __getattr__ -> hasattr(self, 'output') returns True
        if hasattr(self.__class__, 'output'):
            self._deprecated('output', 'Use `save()` or `save_raw()` instead.', stack=5)
            return getattr(self, 'output')()
        return inkex.base.InkscapeExtension.save_raw(self, ret)

    def uniqueId(self, old_id, make_new_id=True):
        self._deprecated('uniqueId', _('{} is now a method in the SvgDocumentElement class. '
                                       ' Use `self.svg.get_unique_id(old_id)` instead.'))
        return self.svg.get_unique_id(old_id)

    def getDocumentWidth(self):
        self._deprecated('getDocumentWidth', _('{} is now a property of the SvgDocumentElement class. '
                                               'Use `self.svg.width` instead.'))
        return self.svg.get('width')

    def getDocumentHeight(self):
        self._deprecated('getDocumentHeight', _('{} is now a property of the SvgDocumentElement class. '
                                                'Use `self.svg.height` instead.'))
        return self.svg.get('height')

    def getDocumentUnit(self):
        self._deprecated('getDocumentUnit', _('{} is now a property of the SvgDocumentElement class. '
                                              'Use `self.svg.unit` instead.'))
        return self.svg.unit

    def unittouu(self, string):
        self._deprecated('unittouu', _('{} is now a method in the SvgDocumentElement class. '
                                       'Use `self.svg.unittouu(str)` instead.'))
        return self.svg.unittouu(string)

    def uutounit(self, val, unit):
        self._deprecated('uutounit', _('{} is now a method in the SvgDocumentElement class. '
                                       'Use `self.svg.uutounit(value, unit)` instead.'))
        return self.svg.uutounit(val, unit)

    def addDocumentUnit(self, value):
        self._deprecated('addDocumentUnit', _('{} is now a method in the SvgDocumentElement class. '
                                              'Use `self.svg.add_unit(value)` instead.'))
        return self.svg.add_unit(value)

class Effect(SvgThroughMixin, DeprecatedEffect, InkscapeExtension):
    """An Inkscape effect, takes SVG in and outputs SVG"""
    pass

def deprecate(func):
    """Function decorator for deprecation functions which have a one-liner
    equivalent in the new API. The one-liner has to passed as a string
    to the decorator.

    >>> @deprecate
    >>> def someOldFunction(*args):
    >>>     '''Example replacement code someNewFunction('foo', ...)'''
    >>>     someNewFunction('foo', *args)

    Or if the args API is the same:

    >>> someOldFunction = deprecate(someNewFunction)

    """

    def _inner(*args, **kwargs):
        _deprecated('{0.__module__}.{0.__name__} -> {0.__doc__}'.format(func), stack=2)
        return func(*args, **kwargs)
    _inner.__name__ = func.__name__
    if func.__doc__:
        _inner.__doc__ = "Deprecated -> " + func.__doc__
    return _inner

class DeprecatedDict(dict):
    @deprecate
    def __getitem__(self, key):
        return super(DeprecatedDict, self).__getitem__(key)

    @deprecate
    def __iter__(self):
        return super(DeprecatedDict, self).__iter__()

# legacy inkex members

class lazyproxy(object):
    """Proxy, use as decorator on a function with provides the wrapped object.
    The decorated function is called when a member is accessed on the proxy.
    """
    def __init__(self, getwrapped):
        '''
        :param getwrapped: Callable which returns the wrapped object
        '''
        self._getwrapped = getwrapped

    def __getattr__(self, name):
        return getattr(self._getwrapped(), name)

    def __call__(self, *args, **kwargs):
        return self._getwrapped()(*args, **kwargs)

@lazyproxy
def optparse():
    _deprecated('inkex.optparse was removed, use "import optparse"', stack=3)
    import optparse as wrapped
    return wrapped

@lazyproxy
def etree():
    _deprecated('inkex.etree was removed, use "from lxml import etree"', stack=3)
    from lxml import etree as wrapped
    return wrapped

@lazyproxy
def InkOption():
    import optparse
    class wrapped(optparse.Option):
        TYPES = optparse.Option.TYPES + ("inkbool", )
        TYPE_CHECKER = dict(optparse.Option.TYPE_CHECKER)
        TYPE_CHECKER["inkbool"] = lambda _1, _2, v: str(v).capitalize() == 'True'
    return wrapped

@lazyproxy
def localize():
    _deprecated('inkex.localize was moved to inkex.localization.localize.', stack=3)
    from .localization import localize as wrapped
    return wrapped

def are_near_relative(a, b, eps):
    _deprecated('inkex.are_near_relative was moved to '
                'inkex.units.are_near_relative', stack=2)
    return inkex.units.are_near_relative(a, b, eps)

def debug(what):
    _deprecated('inkex.debug was moved to inkex.utils.debug.', stack=2)
    return inkex.utils.debug(what)

# legacy inkex members <= 0.48.x

def unittouu(string):
    _deprecated('inkex.unittouu is now a method in the SvgDocumentElement class. '
            'Use `self.svg.unittouu(str)` instead.', stack=2)
    return inkex.units.convert_unit(string, 'px')

# optparse.Values.ensure_value

def ensure_value(self, attr, value):
    _deprecated('Effect().options.ensure_value was removed.', stack=2)
    if getattr(self, attr, None) is None:
        setattr(self, attr, value)
    return getattr(self, attr)

argparse.Namespace.ensure_value = ensure_value # type: ignore

@deprecate
def zSort(inNode, idList):
    """self.svg.get_z_selected()"""
    sortedList = []
    theid = inNode.get("id")
    if theid in idList:
        sortedList.append(theid)
    for child in inNode:
        if len(sortedList) == len(idList):
            break
        sortedList += zSort(child, idList)
    return sortedList

class DeprecatedSvgMixin(object):
    """Mixin which adds deprecated API elements to the SvgDocumentElement"""
    @property
    def selected(self):
        """svg.selection"""
        return self.selection

    @deprecate
    def set_selected(self, *ids):
        """svg.selection.set(*ids)"""
        return self.selection.set(*ids)

    @deprecate
    def get_z_selected(self):
        """svg.selection.paint_order()"""
        return self.selection.paint_order()

    @deprecate
    def get_selected(self, *types):
        """svg.selection.filter(*types).values()"""
        return self.selection.filter(*types).values()

    @deprecate
    def get_selected_or_all(self, *types):
        """Set select_all = True in extension class"""
        if not self.selection:
            self.selection.set_all()
        return self.selection.filter(*types)

    @deprecate
    def get_selected_bbox(self):
        """selection.bounding_box()"""
        return self.selection.bounding_box()

    @deprecate
    def get_first_selected(self, *types):
        """selection.filter(*types).first() or [0] if you'd like an error"""
        return self.selection.filter(*types).first()


# This can't be handled as a mixin class because of circular importing.
def description(self, value):
    """elem.desc = value"""
    self.desc = value
BaseElement.description = deprecate(description)



# -*- coding: utf-8 -*-
#
# Copyright (c) 2020 Martin Owens <doctormo@gmail.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc.,Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
When elements are selected, these structures provide an advanced API.
"""

from collections import OrderedDict
from ._utils import natural_sort_key
from ._base import BaseElement

class ElementList(OrderedDict):
    """
    A list of elements, selected by id, iterator or xpath

    This may look like a dictionary, but it is really a list of elements.
    The default iterator is the element objects themselves (not keys) and it is
    possible to key elements by their numerical index.

    It is also possible to look up items by their id and the element object itself.
    """
    def __init__(self, svg, _iter=None):
        self.svg = svg
        self.ids = OrderedDict()
        super().__init__()
        if _iter is not None:
            self.set(*list(_iter))

    def __iter__(self):
        return self.values().__iter__()

    def __getitem__(self, key):
        return super().__getitem__(self._to_key(key))

    def __contains__(self, key):
        return super().__contains__(self._to_key(key))

    def __setitem__(self, orig_key, elem):
        from ._base import BaseElement
        if orig_key != elem and orig_key != elem.get('id'):
            raise ValueError(f"Refusing to set bad key in ElementList {orig_key}")
        if isinstance(elem, str):
            key = elem
            elem = self.svg.getElementById(elem, literal=True)
            if elem is None:
                return
        if isinstance(elem, BaseElement):
            # Selection is a list of elements to select
            key = elem.xml_path
            element_id = elem.get('id')
            if element_id is not None:
                self.ids[element_id] = key
            super().__setitem__(key, elem)
        else:
            kind = type(elem).__name__
            raise ValueError(f"Unknown element type: {kind}")

    def _to_key(self, key, default=None):
        """Takes a key (id, element, etc) and returns an xml_path key"""
        from ._base import BaseElement
        if self and key is None:
            key = default
        if isinstance(key, int):
            return list(self.keys())[key]
        elif isinstance(key, BaseElement):
            return key.xml_path
        elif isinstance(key, str) and key[0] != '/':
            return self.ids.get(key, key)
        return key

    def clear(self):
        """Also clear ids"""
        self.ids.clear()
        super().clear()

    def set(self, *ids):
        """
        Sets the currently selected elements to these ids, any existing
        selection is cleared.

        Arguments a list of element ids, element objects or
            a single xpath expression starting with "//".

        All element objects must have an id to be correctly set.

        >>> selection.set("rect123", "path456", "text789")
        >>> selection.set(elem1, elem2, elem3)
        >>> selection.set("//rect")
        """
        self.clear()
        self.add(*ids)

    def pop(self, key=None):
        """Remove the key item or remove the last item selected"""
        item = super().pop(self._to_key(key, default=-1))
        self.ids.pop(item.get('id'))
        return item

    def add(self, *ids):
        """Like set() but does not clear first"""
        # Allow selecting of xpath elements directly
        if len(ids) == 1 and isinstance(ids[0], str) and ids[0].startswith('//'):
            ids = self.svg.xpath(ids[0])

        for elem in ids:
            self[elem] = elem # This doesn't matter

    def paint_order(self):
        """Get the selected elements by z-order (stacking order), ordered from bottom to top"""
        new_list = ElementList(self.svg)
        # the elements are stored with their xpath index, so a natural sort order 
        # '3' < '20' < '100' has to be applied
        new_list.set(*[elem for _, elem in sorted(self.items(), key=lambda x: natural_sort_key(x[0]))])
        return new_list

    def filter(self, *types):
        """Filter selected elements of the given type, returns a new SelectedElements object"""
        return ElementList(self.svg, [e for e in self if not types or isinstance(e, types)])

    def get(self, *types):
        """Like filter, but will enter each element searching for any child of the given types"""
        def _recurse(elem):
            if not types or isinstance(elem, types):
                yield elem
            for child in elem:
                yield from _recurse(child)
        return ElementList(self.svg, [r for e in self for r in _recurse(e) if isinstance(r, (BaseElement, str))])

    def id_dict(self):
        """For compatibility, return regular dictionary of id -> element pairs"""
        return dict([(eid, self[xid]) for eid, xid in self.ids.items()])

    def bounding_box(self):
        """
        Gets a :class:`inkex.transforms.BoundingBox` object for the selected items.

        Text objects have a bounding box without width or height that only
        reflects the coordinate of their anchor. If a text object is a part of
        the selection's boundary, the bounding box may be inaccurate.

        When no object is selected or when the object's location cannot be
        determined (e.g. empty group or layer), all coordinates will be None.
        """
        return sum([elem.bounding_box() for elem in self], None)

    def first(self):
        """Returns the first item in the selected list"""
        for elem in self:
            return elem
        return None



# coding=utf-8
#
# Copyright (C) 2010 Martin Owens
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA.
#
"""
Some basic common code shared between EAN and UCP generators.
"""

from .Base import Barcode, TEXT_POS_TOP

try:
    from typing import Optional, List, Dict
except ImportError:
    pass

MAPPING = [
    # Left side of barcode Family '0'
    ["0001101", "0011001", "0010011", "0111101", "0100011",
     "0110001", "0101111", "0111011", "0110111", "0001011"],
    # Left side of barcode Family '1' and flipped to right side.
    ["0100111", "0110011", "0011011", "0100001", "0011101",
     "0111001", "0000101", "0010001", "0001001", "0010111"],
]
# This chooses which of the two encodings above to use.
FAMILIES = ('000000', '001011', '001101', '001110', '010011',
            '011001', '011100', '010101', '010110', '011010')


class EanBarcode(Barcode):
    """Simple base class for all EAN type barcodes"""
    lengths = None # type: Optional[List[int]]
    length = None # type: Optional[int]
    checks = [] # type: List[int]
    extras = {} # type: Dict[int, str] 
    magic = 10
    guard_bar = '202'
    center_bar = '02020'

    def intarray(self, number):
        """Convert a string of digits into an array of ints"""
        return [int(i) for i in number]

    def encode_interleaved(self, family, number, fams=FAMILIES):
        """Encode any side of the barcode, interleaved"""
        result = []
        encset = self.intarray(fams[family])
        for i in range(len(number)):
            thismap = MAPPING[encset[i]]
            result.append(thismap[number[i]])
        return result

    def encode_right(self, number):
        """Encode the right side of the barcode, non-interleaved"""
        result = []
        for num in number:
            # The right side is always the reverse of the left's family '1'
            result.append(MAPPING[1][num][::-1])
        return result

    def encode_left(self, number):
        """Encode the left side of the barcode, non-interleaved"""
        result = []
        for num in number:
            result.append(MAPPING[0][num])
        return result

    def space(self, *spacing):
        """Space out an array of numbers"""
        result = ''
        for space in spacing:
            if isinstance(space, list):
                for i in space:
                    result += str(i)
            elif isinstance(space, int):
                result += ' ' * space
        return result

    def get_lengths(self):
        """Return a list of acceptable lengths"""
        if self.length:
            return [self.length]
        return self.lengths[:]

    def encode(self, code):
        """Encode any EAN barcode"""
        code = code.replace(' ', '').strip()
        guide = code.endswith('>')
        code = code.strip('>')

        if not code.isdigit():
            return self.error(code, 'Not a Number, must be digits 0-9 only')
        lengths = self.get_lengths() + self.checks

        # Allow extra barcodes after the first one
        if len(code) not in lengths:
            for extra in self.extras:
                sep = len(code) - extra
                if sep in lengths:
                    # Generate a barcode along side this one.
                    self.add_extra_barcode(self.extras[extra], text=code[sep:],
                                           x=self.pos_x + 400 * self.scale, text_pos=TEXT_POS_TOP)
                    code = code[:sep]

        if len(code) not in lengths:
            return self.error(code, 'Wrong size {:d}, must be {} digits'.format(len(code), ', '.join([str(length) for length in lengths])))

        if self.checks:
            if len(code) not in self.checks:
                code = self.append_checksum(code)
            elif not self.verify_checksum(code):
                return self.error(code, 'Checksum failed, omit for new sum')
        return self._encode(self.intarray(code), guide=guide)

    def _encode(self, num, guide=False):
        """
        Write your EAN encoding function, it's passed in an array of int and
        it should return a string on 1 and 0 for black and white parts
        """
        raise NotImplementedError("_encode should be provided by parent EAN")

    def enclose(self, left, right=()):
        """Standard Enclosure"""
        parts = [self.guard_bar] + left
        parts.append(self.center_bar)
        parts += list(right) + [self.guard_bar]
        return ''.join(parts)

    def get_checksum(self, num):
        """Generate a UPCA/EAN13/EAN8 Checksum"""
        # Left to right,checksum based on first digits.
        total = sum([int(n) * (3, 1)[x % 2] for x, n in enumerate(num[::-1])])
        # Modulous result to a single digit checksum
        checksum = self.magic - (total % self.magic)
        if checksum < 0 or checksum >= self.magic:
            return '0'
        return str(checksum)

    def append_checksum(self, number):
        """Apply the checksum to a short number"""
        return number + self.get_checksum(number)

    def verify_checksum(self, number):
        """Verify any checksum"""
        return self.get_checksum(number[:-1]) == number[-1]



#!/usr/bin/env python
"""Increase satuation"""

import inkex

class MoreSaturation(inkex.ColorExtension):
    """Increase saturation of selected objects"""
    def modify_color(self, name, color):
        color.saturation += int(0.05 * 255.0)
        return color

if __name__ == '__main__':
    MoreSaturation().run()



#!/usr/bin/env python
"""
Convert PNG to WebP using Raster Output extension.
"""

import inkex

class WebpOutput(inkex.RasterOutputExtension):
    def add_arguments(self, pars):
        pars.add_argument('--tab')
        pars.add_argument('--quality', type=int, default=75)
        pars.add_argument('--speed', type=int, default=3)
        pars.add_argument('--lossless', type=inkex.Boolean, default=True)

    def save(self, stream):
        self.img.save(stream,
            format='webp',
            quality=self.options.quality,
            lossless=self.options.lossless,
            method=self.options.speed)

if __name__ == '__main__':
    WebpOutput().run()



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2009 Aurelio A. Heckert, aurium (a) gmail dot com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
Interpolation of attributes in selected objects or group's children.
"""

import inkex

class InterpAttG(inkex.EffectExtension):
    """
    This effect applies a value for any interpolatable attribute for all
    elements inside the selected group or for all elements in a multiple selection.
    """
    def __init__(self):
        super(InterpAttG, self).__init__()
        self.arg_parser.add_argument(
            "-a", "--att", type=str, dest="att", default="fill",
            help="Attribute to be interpolated.")
        self.arg_parser.add_argument(
            "-o", "--att-other", type=str, dest="att_other",
            help="Other attribute (for a limited UI).")
        self.arg_parser.add_argument(
            "-t", "--att-other-type", type=str, dest="att_other_type",
            help="The other attribute type.")
        self.arg_parser.add_argument(
            "-w", "--att-other-where", type=str, dest="att_other_where",
            help="That is a tag attribute or a style attribute?")
        self.arg_parser.add_argument(
            "-s", "--start-val", type=str, dest="start_val", default="#F00",
            help="Initial interpolation value.")
        self.arg_parser.add_argument(
            "-e", "--end-val", type=str, dest="end_val", default="#00F",
            help="End interpolation value.")
        self.arg_parser.add_argument(
            "-u", "--unit", type=str, dest="unit", default="color",
            help="Values unit.")
        self.arg_parser.add_argument(
            "--zsort", type=inkex.Boolean, dest="zsort", default=True,
            help="use z-order instead of selection order")
        self.arg_parser.add_argument(
            "--tab", type=str, dest="tab",
            help="The selected UI-tab when OK was pressed")

    def get_color_steps(self, total):
        """Get the color value, returning the start color and a single increment step"""
        start_value = inkex.Color(self.options.start_val)
        end_value = inkex.Color(self.options.end_val)

        color_inc = [
            (end_value[v] - start_value[v]) / float(total - 1)
            for v in range(3)]

        return start_value, color_inc

    def get_number_steps(self, total):
        """Get the number value, returning the start float and a single increment step"""
        start_value = self.options.start_val.replace(",", ".")
        end_value = self.options.end_val.replace(",", ".")
        unit = self.options.unit

        if unit != 'none':
            start_value = self.svg.unittouu(start_value + unit)
            end_value = self.svg.unittouu(end_value + unit)

        try:
            start_value = float(start_value)
            end_value = float(end_value)
        except ValueError:
            inkex.errormsg(
                _("Bad values for a number field: {}, {}.".format(start_value, end_value)))
            return 0, 0

        val_inc = (end_value - start_value) / float(total - 1)
        return start_value, val_inc

    def get_elements(self):
        """Returns a list of elements to work on"""
        if not self.svg.selection:
            return []

        if len(self.svg.selection) > 1:
            # multiple selection
            if self.options.zsort:
                return list(self.svg.selection.paint_order().values())
            return list(self.svg.selected.values())

        # must be a group
        node = self.svg.selection.filter(inkex.Group).first()
        return list(node) or []

    def effect(self):
        if self.options.att == 'other':
            if self.options.att_other is not None:
                inte_att = self.options.att_other
            else:
                inkex.errormsg(_("You selected 'Other'. Please enter an attribute to interpolate."))
                return

            inte_att_type = self.options.att_other_type
            where = self.options.att_other_where
        else:
            inte_att = self.options.att
            inte_att_type = 'float'
            if inte_att in ('width', 'height'):
                where = 'tag'
            elif inte_att in ('scale', 'trans-x', 'trans-y'):
                where = 'transform'
            elif inte_att == 'opacity':
                where = 'style'
            elif inte_att in ('fill', 'stroke'):
                inte_att_type = 'color'
                where = 'style'

        collection = self.get_elements()

        if not collection:
            inkex.errormsg(_('There is no selection to interpolate'))
            return False

        if inte_att_type == 'color':
            cur, inc = self.get_color_steps(len(collection))
        else:
            cur, inc = self.get_number_steps(len(collection))

        for node in collection:
            if inte_att_type == 'color':
                val = inkex.Color([int(cur[i]) for i in range(3)])
            elif inte_att_type == 'float':
                val = cur
            elif inte_att_type == 'int':
                val = int(round(cur))
            else:
                raise KeyError("Unknown attr type: {}".format(inte_att_type))

            if where == 'style':
                node.style[inte_att] = str(val)
            elif where == 'transform':
                if inte_att == 'trans-x':
                    node.transform.add_translate(val, 0)
                elif inte_att == 'trans-y':
                    node.transform.add_translate(0, val)
                elif inte_att == 'scale':
                    node.transform.add_scale(val)
            elif where == 'tag':
                node.set(inte_att, str(val))
            else:
                raise KeyError("Unknown update {}".format(where))

            if inte_att_type == 'color':
                cur = [cur[i] + inc[i] for i in range(3)]
            else:
                cur += inc

        return True

if __name__ == '__main__':
    InterpAttG().run()



# coding=utf-8
#
# Copyright (C) 2005 Aaron Spike, aaron@ekips.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

import math
import random


class pTurtle:
    """A Python path turtle"""

    def __init__(self, home=(0, 0)):
        self.__home = [home[0], home[1]]
        self.__pos = self.__home[:]
        self.__heading = -90
        self.__path = ""
        self.__draw = True
        self.__new = True

    def forward(self, mag):
        self.setpos((self.__pos[0] + math.cos(math.radians(self.__heading)) * mag,
                     self.__pos[1] + math.sin(math.radians(self.__heading)) * mag))

    def backward(self, mag):
        self.setpos((self.__pos[0] - math.cos(math.radians(self.__heading)) * mag,
                     self.__pos[1] - math.sin(math.radians(self.__heading)) * mag))

    def right(self, deg):
        self.__heading -= deg

    def left(self, deg):
        self.__heading += deg

    def penup(self):
        self.__draw = False
        self.__new = False

    def pendown(self):
        if not self.__draw:
            self.__new = True
        self.__draw = True

    def pentoggle(self):
        if self.__draw:
            self.penup()
        else:
            self.pendown()

    def home(self):
        self.setpos(self.__home)

    def clean(self):
        self.__path = ''

    def clear(self):
        self.clean()
        self.home()

    def setpos(self, arg):
        if self.__new:
            self.__path += "M" + ",".join([str(i) for i in self.__pos])
            self.__new = False
        self.__pos = arg
        if self.__draw:
            self.__path += "L" + ",".join([str(i) for i in self.__pos])

    def getpos(self):
        return self.__pos[:]

    def setheading(self, deg):
        self.__heading = deg

    def getheading(self):
        return self.__heading

    def sethome(self, arg):
        self.__home = list(arg)

    def getPath(self):
        return self.__path

    def rtree(self, size, minimum, pt=False):
        if size < minimum:
            return
        self.fd(size)
        turn = random.uniform(20, 40)
        self.lt(turn)
        self.rtree(size * random.uniform(0.5, 0.9), minimum, pt)
        self.rt(turn)
        turn = random.uniform(20, 40)
        self.rt(turn)
        self.rtree(size * random.uniform(0.5, 0.9), minimum, pt)
        self.lt(turn)
        if pt:
            self.pu()
        self.bk(size)
        if pt:
            self.pd()

    fd = forward
    bk = backward
    rt = right
    lt = left
    pu = penup
    pd = pendown



#!/usr/bin/env python
# coding=utf-8
#
# Copyright (C) 2007 Peter Lewerin, peter.lewerin@tele2.se
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
"""
An Inkscape effect for adding CAD style dimensions to selected objects
in a drawing.

It uses the selection's bounding box, so if the bounding box has empty
space in the x- or y-direction (such as with some stars) the results
will look strange.  Strokes might also overlap the edge of the
bounding box.

The dimension arrows aren't measured: use the "Visualize Path/Measure
Path" effect to add measurements.

This code contains snippets from existing effects in the Inkscape
extensions library, and marker data from markers.svg.
"""

import inkex
from inkex import Group, Marker, PathElement

import pathmodifier

class Dimension(pathmodifier.PathModifier):
    """Add dimensions as a path modifier"""
    def add_arguments(self, pars):
        pars.add_argument("--xoffset", type=float, default=100.0,\
            help="x offset of the vertical dimension arrow")
        pars.add_argument("--yoffset", type=float, default=100.0,\
            help="y offset of the horizontal dimension arrow")
        pars.add_argument("--type", default="geometric", help="Bounding box type")

    def add_marker(self, name, rotate):
        """Create a marker in the defs of the svg"""
        marker = Marker()
        marker.set('id', name)
        marker.set('orient', 'auto')
        marker.set('refX', '0.0')
        marker.set('refY', '0.0')
        marker.set('style', 'overflow:visible')
        marker.set('inkscape:stockid', name)
        self.svg.defs.append(marker)

        arrow = PathElement(d='M 0.0,0.0 L 5.0,-5.0 L -12.5,0.0 L 5.0,5.0 L 0.0,0.0 z ')
        if rotate:
            arrow.set('transform', 'scale(0.8) rotate(180) translate(12.5,0)')
        else:
            arrow.set('transform', 'scale(0.8) translate(12.5,0)')
        arrow.set('style', 'fill-rule:evenodd;stroke:#000000;stroke-width:1.0pt;marker-start:none')
        marker.append(arrow)

    def horz_line(self, y, xlat, bbox):
        """Create a horzontal line"""
        line = PathElement()
        x1 = bbox.left - xlat[0] * self.options.xoffset
        x2 = bbox.right
        y1 = y - xlat[1] * self.options.yoffset
        line.set('d', 'M %f %f H %f' % (x1, y1, x2))
        return line

    def vert_line(self, x, xlat, bbox):
        """Create a vertical line"""
        line = PathElement()
        x = x - xlat[0] * self.options.xoffset
        y1 = bbox.top - xlat[1] * self.options.yoffset
        y2 = bbox.bottom
        line.set('d', 'M %f %f V %f' % (x, y1, y2))
        return line

    def effect(self):
        scale = self.svg.unittouu('1px')  # convert to document units
        self.options.xoffset *= scale
        self.options.yoffset *= scale

        if not self.svg.selected:
            raise inkex.AbortExtension("Please select an object")
        if self.options.type == "geometric":
            bbox = self.svg.selection.bounding_box()
        else:
            bbox = self.svg.selection.first().bounding_box()

        layer = self.svg.get_current_layer()

        self.add_marker('Arrow1Lstart', False)
        self.add_marker('Arrow1Lend', True)

        group = Group()
        layer.append(group)
        group.set('fill', 'none')
        group.set('stroke', 'black')

        line = self.horz_line(bbox.top, [0, 1], bbox)
        line.set('marker-start', 'url(#Arrow1Lstart)')
        line.set('marker-end', 'url(#Arrow1Lend)')
        line.set('stroke-width', str(scale))
        group.append(line)

        line = self.vert_line(bbox.left, [0, 2], bbox)
        line.set('stroke-width', str(0.5 * scale))
        group.append(line)

        line = self.vert_line(bbox.right, [0, 2], bbox)
        line.set('stroke-width', str(0.5 * scale))
        group.append(line)

        line = self.vert_line(bbox.left, [1, 0], bbox)
        line.set('marker-start', 'url(#Arrow1Lstart)')
        line.set('marker-end', 'url(#Arrow1Lend)')
        line.set('stroke-width', str(scale))
        group.append(line)

        line = self.horz_line(bbox.top, [2, 0], bbox)
        line.set('stroke-width', str(0.5 * scale))
        group.append(line)

        line = self.horz_line(bbox.bottom, [2, 0], bbox)
        line.set('stroke-width', str(0.5 * scale))
        group.append(line)

        for node in self.svg.selected.values():
            group.append(node)

        layer.append(group)
        return None


if __name__ == '__main__':
    Dimension().run()
